# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/Mesh/MeshVTK.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/pyleecan/tree/master/pyleecan/Methods/Mesh/MeshVTK
"""

from os import linesep
from sys import getsizeof
from logging import getLogger
from ._check import check_var, raise_
from ..Functions.get_logger import get_logger
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from .Mesh import Mesh

# Import all class method
# Try/catch to remove unnecessary dependencies in unused method
try:
    from ..Methods.Mesh.MeshVTK.get_mesh_pv import get_mesh_pv
except ImportError as error:
    get_mesh_pv = error

try:
    from ..Methods.Mesh.MeshVTK.get_point import get_point
except ImportError as error:
    get_point = error

try:
    from ..Methods.Mesh.MeshVTK.get_cell import get_cell
except ImportError as error:
    get_cell = error

try:
    from ..Methods.Mesh.MeshVTK.get_normals import get_normals
except ImportError as error:
    get_normals = error

try:
    from ..Methods.Mesh.MeshVTK.get_surf import get_surf
except ImportError as error:
    get_surf = error

try:
    from ..Methods.Mesh.MeshVTK.get_cell_area import get_cell_area
except ImportError as error:
    get_cell_area = error

try:
    from ..Methods.Mesh.MeshVTK.convert import convert
except ImportError as error:
    convert = error

try:
    from ..Methods.Mesh.MeshVTK.as_dict import as_dict
except ImportError as error:
    as_dict = error


from cloudpickle import dumps, loads
from ._check import CheckTypeError

try:
    from pyvista.core.pointset import PointGrid
except ImportError:
    PointGrid = ImportError
from cloudpickle import dumps, loads
from ._check import CheckTypeError

try:
    from pyvista.core.pointset import PolyData
except ImportError:
    PolyData = ImportError
from ._check import InitUnKnowClassError


class MeshVTK(Mesh):
    """Gather the mesh storage format"""

    VERSION = 1

    # Check ImportError to remove unnecessary dependencies in unused method
    # cf Methods.Mesh.MeshVTK.get_mesh_pv
    if isinstance(get_mesh_pv, ImportError):
        get_mesh_pv = property(
            fget=lambda x: raise_(
                ImportError("Can't use MeshVTK method get_mesh_pv: " + str(get_mesh_pv))
            )
        )
    else:
        get_mesh_pv = get_mesh_pv
    # cf Methods.Mesh.MeshVTK.get_point
    if isinstance(get_point, ImportError):
        get_point = property(
            fget=lambda x: raise_(
                ImportError("Can't use MeshVTK method get_point: " + str(get_point))
            )
        )
    else:
        get_point = get_point
    # cf Methods.Mesh.MeshVTK.get_cell
    if isinstance(get_cell, ImportError):
        get_cell = property(
            fget=lambda x: raise_(
                ImportError("Can't use MeshVTK method get_cell: " + str(get_cell))
            )
        )
    else:
        get_cell = get_cell
    # cf Methods.Mesh.MeshVTK.get_normals
    if isinstance(get_normals, ImportError):
        get_normals = property(
            fget=lambda x: raise_(
                ImportError("Can't use MeshVTK method get_normals: " + str(get_normals))
            )
        )
    else:
        get_normals = get_normals
    # cf Methods.Mesh.MeshVTK.get_surf
    if isinstance(get_surf, ImportError):
        get_surf = property(
            fget=lambda x: raise_(
                ImportError("Can't use MeshVTK method get_surf: " + str(get_surf))
            )
        )
    else:
        get_surf = get_surf
    # cf Methods.Mesh.MeshVTK.get_cell_area
    if isinstance(get_cell_area, ImportError):
        get_cell_area = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use MeshVTK method get_cell_area: " + str(get_cell_area)
                )
            )
        )
    else:
        get_cell_area = get_cell_area
    # cf Methods.Mesh.MeshVTK.convert
    if isinstance(convert, ImportError):
        convert = property(
            fget=lambda x: raise_(
                ImportError("Can't use MeshVTK method convert: " + str(convert))
            )
        )
    else:
        convert = convert
    # cf Methods.Mesh.MeshVTK.as_dict
    if isinstance(as_dict, ImportError):
        as_dict = property(
            fget=lambda x: raise_(
                ImportError("Can't use MeshVTK method as_dict: " + str(as_dict))
            )
        )
    else:
        as_dict = as_dict
    # save and copy methods are available in all object
    save = save
    copy = copy
    # get_logger method is available in all object
    get_logger = get_logger

    def __init__(
        self,
        mesh=None,
        is_pyvista_mesh=False,
        format="vtk",
        path="",
        name="mesh",
        surf=None,
        is_vtk_surf=False,
        surf_path="",
        surf_name="surf",
        label=None,
        dimension=2,
        init_dict=None,
        init_str=None,
    ):
        """Constructor of the class. Can be use in three ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for pyleecan type, -1 will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary with property names as keys
        - __init__ (init_str = s) s must be a string
        s is the file path to load

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for pyleecan Object"""

        if init_str is not None:  # Load from a file
            init_dict = load_init_dict(init_str)[1]
        if init_dict is not None:  # Initialisation by dict
            assert type(init_dict) is dict
            # Overwrite default value with init_dict content
            if "mesh" in list(init_dict.keys()):
                mesh = init_dict["mesh"]
            if "is_pyvista_mesh" in list(init_dict.keys()):
                is_pyvista_mesh = init_dict["is_pyvista_mesh"]
            if "format" in list(init_dict.keys()):
                format = init_dict["format"]
            if "path" in list(init_dict.keys()):
                path = init_dict["path"]
            if "name" in list(init_dict.keys()):
                name = init_dict["name"]
            if "surf" in list(init_dict.keys()):
                surf = init_dict["surf"]
            if "is_vtk_surf" in list(init_dict.keys()):
                is_vtk_surf = init_dict["is_vtk_surf"]
            if "surf_path" in list(init_dict.keys()):
                surf_path = init_dict["surf_path"]
            if "surf_name" in list(init_dict.keys()):
                surf_name = init_dict["surf_name"]
            if "label" in list(init_dict.keys()):
                label = init_dict["label"]
            if "dimension" in list(init_dict.keys()):
                dimension = init_dict["dimension"]
        # Set the properties (value check and convertion are done in setter)
        self.mesh = mesh
        self.is_pyvista_mesh = is_pyvista_mesh
        self.format = format
        self.path = path
        self.name = name
        self.surf = surf
        self.is_vtk_surf = is_vtk_surf
        self.surf_path = surf_path
        self.surf_name = surf_name
        # Call Mesh init
        super(MeshVTK, self).__init__(label=label, dimension=dimension)
        # The class is frozen (in Mesh init), for now it's impossible to
        # add new properties

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        MeshVTK_str = ""
        # Get the properties inherited from Mesh
        MeshVTK_str += super(MeshVTK, self).__str__()
        MeshVTK_str += "mesh = " + str(self.mesh) + linesep + linesep
        MeshVTK_str += "is_pyvista_mesh = " + str(self.is_pyvista_mesh) + linesep
        MeshVTK_str += 'format = "' + str(self.format) + '"' + linesep
        MeshVTK_str += 'path = "' + str(self.path) + '"' + linesep
        MeshVTK_str += 'name = "' + str(self.name) + '"' + linesep
        MeshVTK_str += "surf = " + str(self.surf) + linesep + linesep
        MeshVTK_str += "is_vtk_surf = " + str(self.is_vtk_surf) + linesep
        MeshVTK_str += 'surf_path = "' + str(self.surf_path) + '"' + linesep
        MeshVTK_str += 'surf_name = "' + str(self.surf_name) + '"' + linesep
        return MeshVTK_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False

        # Check the properties inherited from Mesh
        if not super(MeshVTK, self).__eq__(other):
            return False
        if other.mesh != self.mesh:
            return False
        if other.is_pyvista_mesh != self.is_pyvista_mesh:
            return False
        if other.format != self.format:
            return False
        if other.path != self.path:
            return False
        if other.name != self.name:
            return False
        if other.surf != self.surf:
            return False
        if other.is_vtk_surf != self.is_vtk_surf:
            return False
        if other.surf_path != self.surf_path:
            return False
        if other.surf_name != self.surf_name:
            return False
        return True

    def compare(self, other, name="self"):
        """Compare two objects and return list of differences"""

        if type(other) != type(self):
            return ["type(" + name + ")"]
        diff_list = list()

        # Check the properties inherited from Mesh
        diff_list.extend(super(MeshVTK, self).compare(other, name=name))
        if (other.mesh is None and self.mesh is not None) or (
            other.mesh is not None and self.mesh is None
        ):
            diff_list.append(name + ".mesh None mismatch")
        elif self.mesh is not None:
            diff_list.extend(self.mesh.compare(other.mesh, name=name + ".mesh"))
        if other._is_pyvista_mesh != self._is_pyvista_mesh:
            diff_list.append(name + ".is_pyvista_mesh")
        if other._format != self._format:
            diff_list.append(name + ".format")
        if other._path != self._path:
            diff_list.append(name + ".path")
        if other._name != self._name:
            diff_list.append(name + ".name")
        if (other.surf is None and self.surf is not None) or (
            other.surf is not None and self.surf is None
        ):
            diff_list.append(name + ".surf None mismatch")
        elif self.surf is not None:
            diff_list.extend(self.surf.compare(other.surf, name=name + ".surf"))
        if other._is_vtk_surf != self._is_vtk_surf:
            diff_list.append(name + ".is_vtk_surf")
        if other._surf_path != self._surf_path:
            diff_list.append(name + ".surf_path")
        if other._surf_name != self._surf_name:
            diff_list.append(name + ".surf_name")
        return diff_list

    def __sizeof__(self):
        """Return the size in memory of the object (including all subobject)"""

        S = 0  # Full size of the object

        # Get size of the properties inherited from Mesh
        S += super(MeshVTK, self).__sizeof__()
        S += getsizeof(self.mesh)
        S += getsizeof(self.is_pyvista_mesh)
        S += getsizeof(self.format)
        S += getsizeof(self.path)
        S += getsizeof(self.name)
        S += getsizeof(self.surf)
        S += getsizeof(self.is_vtk_surf)
        S += getsizeof(self.surf_path)
        S += getsizeof(self.surf_name)
        return S

    def _set_None(self):
        """Set all the properties to None (except pyleecan object)"""

        self.mesh = None
        self.is_pyvista_mesh = None
        self.format = None
        self.path = None
        self.name = None
        self.surf = None
        self.is_vtk_surf = None
        self.surf_path = None
        self.surf_name = None
        # Set to None the properties inherited from Mesh
        super(MeshVTK, self)._set_None()

    def _get_mesh(self):
        """getter of mesh"""
        return self._mesh

    def _set_mesh(self, value):
        """setter of mesh"""
        check_var("mesh", value, "PointGrid")
        self._mesh = value

    mesh = property(
        fget=_get_mesh,
        fset=_set_mesh,
        doc=u"""Pyvista object of the mesh (optional)

        :Type: pyvista.core.pointset.PointGrid
        """,
    )

    def _get_is_pyvista_mesh(self):
        """getter of is_pyvista_mesh"""
        return self._is_pyvista_mesh

    def _set_is_pyvista_mesh(self, value):
        """setter of is_pyvista_mesh"""
        check_var("is_pyvista_mesh", value, "bool")
        self._is_pyvista_mesh = value

    is_pyvista_mesh = property(
        fget=_get_is_pyvista_mesh,
        fset=_set_is_pyvista_mesh,
        doc=u"""Store the pyvista object

        :Type: bool
        """,
    )

    def _get_format(self):
        """getter of format"""
        return self._format

    def _set_format(self, value):
        """setter of format"""
        check_var("format", value, "str")
        self._format = value

    format = property(
        fget=_get_format,
        fset=_set_format,
        doc=u"""Format in which the mesh is stored

        :Type: str
        """,
    )

    def _get_path(self):
        """getter of path"""
        return self._path

    def _set_path(self, value):
        """setter of path"""
        check_var("path", value, "str")
        self._path = value

    path = property(
        fget=_get_path,
        fset=_set_path,
        doc=u"""Path where the mesh is stored

        :Type: str
        """,
    )

    def _get_name(self):
        """getter of name"""
        return self._name

    def _set_name(self, value):
        """setter of name"""
        check_var("name", value, "str")
        self._name = value

    name = property(
        fget=_get_name,
        fset=_set_name,
        doc=u"""Name of the mesh file

        :Type: str
        """,
    )

    def _get_surf(self):
        """getter of surf"""
        return self._surf

    def _set_surf(self, value):
        """setter of surf"""
        check_var("surf", value, "PolyData")
        self._surf = value

    surf = property(
        fget=_get_surf,
        fset=_set_surf,
        doc=u"""Pyvista object of the outer surface

        :Type: pyvista.core.pointset.PolyData
        """,
    )

    def _get_is_vtk_surf(self):
        """getter of is_vtk_surf"""
        return self._is_vtk_surf

    def _set_is_vtk_surf(self, value):
        """setter of is_vtk_surf"""
        check_var("is_vtk_surf", value, "bool")
        self._is_vtk_surf = value

    is_vtk_surf = property(
        fget=_get_is_vtk_surf,
        fset=_set_is_vtk_surf,
        doc=u"""Save the surface mesh in a vtk file

        :Type: bool
        """,
    )

    def _get_surf_path(self):
        """getter of surf_path"""
        return self._surf_path

    def _set_surf_path(self, value):
        """setter of surf_path"""
        check_var("surf_path", value, "str")
        self._surf_path = value

    surf_path = property(
        fget=_get_surf_path,
        fset=_set_surf_path,
        doc=u"""Path where the outer surface is stored

        :Type: str
        """,
    )

    def _get_surf_name(self):
        """getter of surf_name"""
        return self._surf_name

    def _set_surf_name(self, value):
        """setter of surf_name"""
        check_var("surf_name", value, "str")
        self._surf_name = value

    surf_name = property(
        fget=_get_surf_name,
        fset=_set_surf_name,
        doc=u"""Name of the outer surface file

        :Type: str
        """,
    )



# -*- coding: utf-8 -*-

from numpy import pi
from ....Classes.Lamination import Lamination


def comp_surfaces(self):
    """Compute the Lamination surfaces (Lamination, Ventilation, Slot).

    Parameters
    ----------
    self : LamSlotMulti
        A LamSlotMulti object

    Returns
    -------
    S_dict: dict
        Lamination surface dictionnary (Slam, Svent, Sslot) [m**2]

    """

    S_dict = Lamination.comp_surfaces(self)
    Sslot = 0
    for slot in self.slot_list:
        Sslot += slot.comp_surface()

    S_dict["Sslot"] = Sslot
    S_dict["Slam"] -= Sslot
    return S_dict



# -*- coding: utf-8 -*-

from numpy import pi
from pyleecan.Classes.Segment import Segment


def comp_magnetization_dict(self, is_north=True):
    """Compute the dictionary of the magnetization direction of the magnets (key=magnet_X, value=angle[rad])
    Mangetization angle with Hole centered on Ox axis

    Parameters
    ----------
    self : HoleM54
        a HoleM54 object
    is_north: True
        True: comp north magnetization, else add pi [rad]

    Returns
    -------
    mag_dict: dict
        magnetization dictionary (key=magnet_X, value=angle[rad])
    """

    mag_dict = dict()

    return mag_dict



from numpy import exp, pi, cos, sin, tan
from ....Functions.Geometry.inter_line_circle import inter_line_circle


def _comp_point_coordinate(self):
    """Compute the point coordinates needed to plot the Slot.

    Parameters
    ----------
    self : HoleM51
        A HoleM51 object

    Returns
    -------
    point_dict: dict
        A dict of the slot coordinates
    """

    Rext = self.get_Rext()

    # comp point coordinate (in complex)
    alpha = self.comp_alpha()

    Wslot = 2 * sin(self.W1 / 2) * (Rext - self.H1)
    L = 0.5 * (Wslot - self.W0) / cos(alpha)  # ||P2,P5||

    # Center of the hole
    Z0 = Rext - self.H0
    Z2 = Z0 + 1j * self.W0 / 2
    Z25 = Z0 - 1j * self.W0 / 2
    Z15 = Z25 - self.H2
    Z1 = Z2 - 1j * self.W2
    Z26 = Z1 - 1j * self.W3
    Z12 = Z2 - self.H2
    Z13 = Z12 - 1j * self.W2
    Z14 = Z13 - 1j * self.W3
    Z11 = Z12 + 1j * tan(alpha / 2) * self.H2
    Z16 = Z15 - 1j * tan(alpha / 2) * self.H2

    # Draw the left side with center P2, and X axis =(P2,P5), Y axis=(P2,P10)
    Z3 = self.W4 * exp(1j * (pi / 2 - alpha)) + Z2
    Z4 = (self.W4 + self.W5) * exp(1j * (pi / 2 - alpha)) + Z2
    Z5 = (Rext - self.H1) * exp(1j * self.W1 / 2)
    Z10 = (1j * self.H2) * exp(1j * (pi / 2 - alpha)) + Z2
    Z9 = (1j * self.H2 + self.W4) * exp(1j * (pi / 2 - alpha)) + Z2
    Z8 = (1j * self.H2 + self.W4 + self.W5) * exp(1j * (pi / 2 - alpha)) + Z2
    Z7 = (1j * self.H2 + L) * exp(1j * (pi / 2 - alpha)) + Z2

    # Draw the right side with center P25, X axis (P25,P23), Y axis(P25,P17)
    Z24 = self.W6 * exp(-1j * (pi / 2 - alpha)) + Z25
    Z23 = (self.W6 + self.W7) * exp(-1j * (pi / 2 - alpha)) + Z25
    Z22 = (Rext - self.H1) * exp(-1j * self.W1 / 2)
    Z17 = (-1j * self.H2) * exp(-1j * (pi / 2 - alpha)) + Z25
    Z18 = (-1j * self.H2 + self.W6) * exp(-1j * (pi / 2 - alpha)) + Z25
    Z19 = (-1j * self.H2 + self.W6 + self.W7) * exp(-1j * (pi / 2 - alpha)) + Z25
    Z20 = (-1j * self.H2 + L) * exp(-1j * (pi / 2 - alpha)) + Z25

    # Z6 is the intersection of the line [Z7,Z10] and Circle centre
    # (0,0) radius Rext - H1
    Zint = inter_line_circle(Z7, Z10, Rext - self.H1)

    # Select the point with Re(Z) > 0
    if Zint[0].real > 0:
        Z6 = Zint[0]
    else:
        Z6 = Zint[1]
    Z21 = Z6.conjugate()

    point_dict = dict()
    point_dict["Z1"] = Z1
    point_dict["Z2"] = Z2
    point_dict["Z3"] = Z3
    point_dict["Z4"] = Z4
    point_dict["Z5"] = Z5
    point_dict["Z6"] = Z6
    point_dict["Z7"] = Z7
    point_dict["Z8"] = Z8
    point_dict["Z9"] = Z9
    point_dict["Z10"] = Z10
    point_dict["Z11"] = Z11
    point_dict["Z12"] = Z12
    point_dict["Z13"] = Z13
    point_dict["Z14"] = Z14
    point_dict["Z15"] = Z15
    point_dict["Z16"] = Z16
    point_dict["Z17"] = Z17
    point_dict["Z18"] = Z18
    point_dict["Z19"] = Z19
    point_dict["Z20"] = Z20
    point_dict["Z21"] = Z21
    point_dict["Z22"] = Z22
    point_dict["Z23"] = Z23
    point_dict["Z24"] = Z24
    point_dict["Z25"] = Z25
    point_dict["Z26"] = Z26

    return point_dict



import matplotlib.pyplot as plt
from numpy import pi, exp

from ....Classes.Arc1 import Arc1
from ....Classes.LamSlot import LamSlot
from ....Classes.Segment import Segment
from ....definitions import config_dict
from ....Functions.Plot import (
    ARROW_COLOR,
    ARROW_WIDTH,
    MAIN_LINE_COLOR,
    MAIN_LINE_STYLE,
    MAIN_LINE_WIDTH,
    P_FONT_SIZE,
    SC_FONT_SIZE,
    SC_LINE_COLOR,
    SC_LINE_STYLE,
    SC_LINE_WIDTH,
    TEXT_BOX,
    plot_quote,
)
from ....Methods import ParentMissingError

MAGNET_COLOR = config_dict["PLOT"]["COLOR_DICT"]["MAGNET_COLOR"]


def plot_schematics(
    self,
    is_default=False,
    is_add_point_label=False,
    is_add_schematics=True,
    is_add_main_line=True,
    type_add_active=True,
    save_path=None,
    is_show_fig=True,
):
    """Plot the schematics of the slot

    Parameters
    ----------
    self : SlotM13
        A SlotM13 object
    is_default : bool
        True: plot default schematics, else use current slot values
    is_add_point_label : bool
        True to display the name of the points (Z1, Z2....)
    is_add_schematics : bool
        True to display the schematics information (W0, H0...)
    is_add_main_line : bool
        True to display "main lines" (slot opening and 0x axis)
    type_add_active : int
        0: No active surface, 1: active surface as winding, 2: active surface as magnet
    save_path : str
        full path including folder, name and extension of the file to save if save_path is not None
    is_show_fig : bool
        To call show at the end of the method
    """

    # Use some default parameter
    if is_default:
        slot = type(self)(Zs=4, W0=0.05, H0=0.02, Hmag=0.02, Wmag=0.04, Rtopm=0.04)
        lam = LamSlot(
            Rint=0.1, Rext=0.135, is_internal=True, is_stator=False, slot=slot
        )
        slot.plot_schematics(
            is_default=False,
            is_add_point_label=is_add_point_label,
            is_add_schematics=is_add_schematics,
            is_add_main_line=is_add_main_line,
            type_add_active=type_add_active,
            save_path=save_path,
            is_show_fig=is_show_fig,
        )
    else:
        # Getting the main plot
        if self.parent is None:
            raise ParentMissingError("Error: The slot is not inside a Lamination")
        lam = self.parent
        lam.plot(alpha=pi / self.Zs, is_show_fig=False)  # center slot on Ox axis
        fig = plt.gcf()
        ax = plt.gca()
        point_dict = self._comp_point_coordinate()
        if self.is_outwards():
            sign = +1
        else:
            sign = -1

        # Adding point label
        if is_add_point_label:
            for name, Z in point_dict.items():
                ax.text(
                    Z.real,
                    Z.imag,
                    name,
                    fontsize=P_FONT_SIZE,
                    bbox=TEXT_BOX,
                )

        # Adding schematics
        if is_add_schematics:
            # W0
            line = Segment(point_dict["Z3"], point_dict["Z2"])
            line.plot(
                fig=fig,
                ax=ax,
                color=ARROW_COLOR,
                linewidth=ARROW_WIDTH,
                label="W0",
                offset_label=-1 * sign * self.Hmag * 0.1 + 1j * self.Hmag * 0.2,
                is_arrow=True,
                fontsize=SC_FONT_SIZE,
            )
            # Wmag
            plot_quote(
                Z1=point_dict["ZM2"],
                Zlim1=point_dict["ZM2"] - sign * 0.5 * self.Hmag,
                Zlim2=point_dict["ZM3"] - sign * 0.5 * self.Hmag,
                Z2=point_dict["ZM3"],
                offset_label=0.25 * self.Hmag,
                fig=fig,
                ax=ax,
                label="Wmag",
            )
            # H0
            line = Segment(point_dict["Z1"], point_dict["Z2"])
            line.plot(
                fig=fig,
                ax=ax,
                color=ARROW_COLOR,
                linewidth=ARROW_WIDTH,
                label="H0",
                offset_label=1j * -0.1 * self.W0,
                is_arrow=True,
                fontsize=SC_FONT_SIZE,
            )
            # Hmag
            Zlim1 = point_dict["ZM4"] - 1j * sign * 0.3 * self.Hmag
            plot_quote(
                Z1=point_dict["ZM4"],
                Zlim1=Zlim1,
                Zlim2=point_dict["ZM0"].real + 1j * Zlim1.imag,
                Z2=point_dict["ZM0"],
                offset_label=1j * 0.1 * self.Wmag,
                fig=fig,
                ax=ax,
                label="Hmag",
            )
            # Rtopm
            line = Segment(point_dict["Zc"], point_dict["ZM0"])
            line.plot(
                fig=fig,
                ax=ax,
                color=ARROW_COLOR,
                linewidth=ARROW_WIDTH,
                label="Rtopm",
                offset_label=sign * self.Rtopm * 0.4 + 1j * 0.1 * self.Wmag,
                is_arrow=True,
                fontsize=SC_FONT_SIZE,
            )

        if is_add_main_line:
            # Ox axis
            line = Segment(0, lam.Rext * 1.5)
            line.plot(
                fig=fig,
                ax=ax,
                color=MAIN_LINE_COLOR,
                linestyle=MAIN_LINE_STYLE,
                linewidth=MAIN_LINE_WIDTH,
            )
            # Top arc
            line = Arc1(
                begin=point_dict["Z1"],
                end=point_dict["Z4"],
                radius=self.get_Rbo(),
                is_trigo_direction=True,
            )
            line.plot(
                fig=fig,
                ax=ax,
                color=MAIN_LINE_COLOR,
                linestyle=MAIN_LINE_STYLE,
                linewidth=MAIN_LINE_WIDTH,
            )
            # Magnet Arc
            line = Segment(point_dict["Zc"], point_dict["ZM2"])
            line.plot(
                fig=fig,
                ax=ax,
                color=MAIN_LINE_COLOR,
                linestyle=MAIN_LINE_STYLE,
                linewidth=MAIN_LINE_WIDTH,
            )
            line = Segment(point_dict["Zc"], point_dict["ZM3"])
            line.plot(
                fig=fig,
                ax=ax,
                color=MAIN_LINE_COLOR,
                linestyle=MAIN_LINE_STYLE,
                linewidth=MAIN_LINE_WIDTH,
            )

        if type_add_active == 1:
            self.plot_active(fig=fig, is_show_fig=False)
        elif type_add_active == 2:
            self.plot_active(
                fig=fig, is_show_fig=False, enforced_default_color=MAGNET_COLOR
            )

        # Zooming and cleaning
        W = self.W0 / 2 * 1.4
        Rint, Rext = self.comp_radius()

        plt.axis("equal")
        ax.set_xlim(Rint, Rext)
        ax.set_ylim(-W, W)
        fig.canvas.set_window_title(type(self).__name__ + " Schematics")
        ax.set_title("")
        ax.get_legend().remove()
        ax.set_axis_off()

        # Save / Show
        if save_path is not None:
            fig.savefig(save_path)
            plt.close()

        if is_show_fig:
            fig.show()



# -*- coding: utf-8 -*-
"""File generated according to PCondType21/gen_list.json
WARNING! All changes made in this file will be lost!
"""
from pyleecan.GUI.Dialog.DMachineSetup.SBar.PCondType21.Ui_PCondType21 import (
    Ui_PCondType21,
)


class Gen_PCondType21(Ui_PCondType21):
    def setupUi(self, PCondType21):
        """Abstract class to update the widget according to the csv doc"""
        Ui_PCondType21.setupUi(self, PCondType21)
        # Setup of w_mat
        txt = self.tr(u"""Material of the conductor""")
        self.w_mat.setWhatsThis(txt)
        self.w_mat.setToolTip(txt)

        # Setup of lf_Hbar
        self.lf_Hbar.validator().setBottom(0)
        txt = self.tr(u"""Bar height""")
        self.lf_Hbar.setWhatsThis(txt)
        self.lf_Hbar.setToolTip(txt)

        # Setup of in_Hbar
        txt = self.tr(u"""Bar height""")
        self.in_Hbar.setWhatsThis(txt)
        self.in_Hbar.setToolTip(txt)

        # Setup of in_Wbar
        txt = self.tr(u"""Bar width""")
        self.in_Wbar.setWhatsThis(txt)
        self.in_Wbar.setToolTip(txt)

        # Setup of lf_Wbar
        self.lf_Wbar.validator().setBottom(0)
        txt = self.tr(u"""Bar width""")
        self.lf_Wbar.setWhatsThis(txt)
        self.lf_Wbar.setToolTip(txt)



class InputError(Exception):
    """ """

    pass


#
# boundary_list = [
#     "MASTER_ROTOR_BOUNDARY",
#     "SLAVE_ROTOR_BOUNDARY",
#     "SB_ROTOR_BOUNDARY",
#     "MASTER_STATOR_BOUNDARY",
#     "SLAVE_STATOR_BOUNDARY",
#     "SB_STATOR_BOUNDARY",
#     "AIRGAP_ARC_BOUNDARY",
#     "VP0_BOUNDARY",
# ]
#
# # dictionary to match FEA boundary conditions (dict values)
# # with line labels (dict keys) that are set in the build_geometry methods
# boundary_prop = dict()
# boundary_prop["int_airgap_line_1"] = "MASTER_ROTOR_BOUNDARY"
# boundary_prop["int_airgap_line_2"] = "SLAVE_ROTOR_BOUNDARY"
# boundary_prop["int_sb_line_1"] = "MASTER_ROTOR_BOUNDARY"
# boundary_prop["int_sb_line_2"] = "SLAVE_ROTOR_BOUNDARY"
# boundary_prop["Lamination_Rotor_Yoke_Side_Right"] = "MASTER_ROTOR_BOUNDARY"
# boundary_prop["Lamination_Rotor_Yoke_Side_Left"] = "SLAVE_ROTOR_BOUNDARY"
# boundary_prop["Shaft_Side_Right"] = "MASTER_ROTOR_BOUNDARY"
# boundary_prop["Shaft_Side_Left"] = "SLAVE_ROTOR_BOUNDARY"
# boundary_prop["int_sb_arc"] = "SB_ROTOR_BOUNDARY"
# boundary_prop["ext_airgap_line_1"] = "MASTER_STATOR_BOUNDARY"
# boundary_prop["ext_airgap_line_2"] = "SLAVE_STATOR_BOUNDARY"
# boundary_prop["ext_sb_line_1"] = "MASTER_STATOR_BOUNDARY"
# boundary_prop["ext_sb_line_2"] = "SLAVE_STATOR_BOUNDARY"
# boundary_prop["airbox_line_1"] = "MASTER_STATOR_BOUNDARY"
# boundary_prop["airbox_line_2"] = "SLAVE_STATOR_BOUNDARY"
# boundary_prop["Lamination_Stator_Yoke_Side_Right"] = "MASTER_STATOR_BOUNDARY"
# boundary_prop["Lamination_Stator_Yoke_Side_Left"] = "SLAVE_STATOR_BOUNDARY"
# boundary_prop["ext_sb_arc"] = "SB_STATOR_BOUNDARY"
# boundary_prop["ext_airgap_arc_copy"] = "AIRGAP_ARC_BOUNDARY"
# boundary_prop["airbox_arc"] = "VP0_BOUNDARY"
#
# # dict for the translation of the actual surface labels into Elmer compatible labels,
# # i.e. max. 30 characters; key: actual label, value: Elmer label
# surface_label = dict()
# surface_label["Shaft"] = "SHAFT"
# surface_label["Lamination_Rotor"] = "ROTOR_LAM"
# surface_label["Lamination_Stator"] = "STATOR_LAM"
# surface_label["Lamination_Rotor_Bore_Radius_Ext"] = "ROTOR_LAM"
# surface_label["Lamination_Stator_Bore_Radius_Int"] = "STATOR_LAM"
# surface_label["Airgap_int"] = "AG_INT"
# surface_label["SlidingBand_int"] = "SB_INT"
# surface_label["SlidingBand_ext"] = "SB_EXT"
# surface_label["Airgap_ext"] = "AG_EXT"
# surface_label["Airbox"] = "AIRBOX"
#
# # TODO: use actual surface labels
# for rr in range(0, 3):
#     for tt in range(0, 3):
#         for ss in range(0, 60):
#             # Windings
#             old_label = "Wind_Stator_R" + str(rr) + "_T" + str(tt) + "_S" + str(ss)
#             new_label = "W_STA_R" + str(rr) + "_T" + str(tt) + "_S" + str(ss)
#             surface_label[old_label] = new_label
#
#
# for rr in range(0, 5):
#     for tt in range(0, 5):
#         for ss in range(0, 20):
#             # Rotor Holes
#             old_label = "Hole_Rotor_R" + str(rr) + "_T" + str(tt) + "_S" + str(ss)
#             new_label = "H_ROTOR_R" + str(rr) + "T" + str(tt) + "S" + str(ss)
#             surface_label[old_label] = new_label
#             # Interior Magnets
#             old_label = (
#                 "HoleMagnet_Rotor_Parallel_N_R"
#                 + str(rr)
#                 + "_T"
#                 + str(tt)
#                 + "_S"
#                 + str(ss)
#             )
#             new_label = "H_MAGNET_ROT_PAR_N_R" + str(rr) + "T" + str(tt) + "S" + str(ss)
#             surface_label[old_label] = new_label
#             old_label = (
#                 "HoleMagnet_Rotor_Parallel_S_R"
#                 + str(rr)
#                 + "_T"
#                 + str(tt)
#                 + "_S"
#                 + str(ss)
#             )
#             new_label = "H_MAGNET_ROT_PAR_S_R" + str(rr) + "T" + str(tt) + "S" + str(ss)
#             surface_label[old_label] = new_label
#             old_label = (
#                 "HoleMagnet_Rotor_Radial_N_R"
#                 + str(rr)
#                 + "_T"
#                 + str(tt)
#                 + "_S"
#                 + str(ss)
#             )
#             new_label = "H_MAGNET_ROT_RAD_N_R" + str(rr) + "T" + str(tt) + "S" + str(ss)
#             surface_label[old_label] = new_label
#             old_label = (
#                 "HoleMagnet_Rotor_Radial_S_R"
#                 + str(rr)
#                 + "_T"
#                 + str(tt)
#                 + "_S"
#                 + str(ss)
#             )
#             new_label = "H_MAGNET_ROT_RAD_S_R" + str(rr) + "T" + str(tt) + "S" + str(ss)
#             surface_label[old_label] = new_label
#             # Surface Magnets
#             old_label = (
#                     "MagnetRotorParallel_N_R"
#                     + str(rr)
#                     + "_T"
#                     + str(tt)
#                     + "_S"
#                     + str(ss)
#             )
#             new_label = "MAGNET_ROT_PAR_N_R" + str(rr) + "T" + str(tt) + "S" + str(ss)
#             surface_label[old_label] = new_label
#             old_label = (
#                     "MagnetRotorParallel_S_R"
#                     + str(rr)
#                     + "_T"
#                     + str(tt)
#                     + "_S"
#                     + str(ss)
#             )
#             new_label = "MAGNET_ROT_PAR_S_R" + str(rr) + "T" + str(tt) + "S" + str(ss)
#             surface_label[old_label] = new_label
#             old_label = (
#                     "MagnetRotorRadial_N_R"
#                     + str(rr)
#                     + "_T"
#                     + str(tt)
#                     + "_S"
#                     + str(ss)
#             )
#             new_label = "MAGNET_ROT_RAD_N_R" + str(rr) + "T" + str(tt) + "S" + str(ss)
#             surface_label[old_label] = new_label
#             old_label = (
#                     "MagnetRotorRadial_S_R"
#                     + str(rr)
#                     + "_T"
#                     + str(tt)
#                     + "_S"
#                     + str(ss)
#             )
#             new_label = "MAGNET_ROT_RAD_S_R" + str(rr) + "T" + str(tt) + "S" + str(ss)
#             surface_label[old_label] = new_label
#
#



# -*- coding: utf-8 -*-

from numpy import pi, power

from ....Methods.Machine.CondType12 import *


def check(self):
    """Check that the Conductor object is correct

    Parameters
    ----------
    self : CondType12
        A CondType12 object

    Returns
    -------
    None

    Raises
    _______
    C12_WireDontFit
        The wire is too big to fit in the conductor
    C12_WireDontFit
        The wires are too big to fit in the conductor
    """

    Swire = power((self.Wwire + 2 * self.Wins_wire) / 2.0, 2) * pi * self.Nwppc
    Scond = self.comp_surface()

    if Swire > Scond:
        if self.Nwppc == 1:
            raise C12_WireDontFit("The wire is too big to fit in the conductor")
        else:
            raise C12_WireDontFit("The wires are too big to fit in the conductor")



# -*- coding: utf-8 -*-

from ....Classes.LamSlotWind import LamSlotWind
from ....Classes.LamSquirrelCage import LamSquirrelCage
from ....Methods.Machine import MachineCheckError


def check(self):
    """Check that the Machine object is correct

    Parameters
    ----------
    self : MachineSCIM
        A MachineSCIM object

    Returns
    -------
    None

    Raises
    _______
    M1C_WrongRotor
        The Rotor of a MachineSCIM must be a LamSquirrelCage
    M1C_WrongStator
        The Stator of a MachineSCIM must be a LamSlotWind
    M1C_SquirrelCagePError
        The stator and the rotor winding must have the same value for p
    M1C_RingTooLarge
        The Ring is larger than the stator internal radius
    """
    # Call Machine check, skip MachineDFIM check
    self.__class__.__bases__[0].__bases__[0].check(self)

    if not isinstance(self.rotor, LamSquirrelCage):
        raise M1C_WrongRotor("The Rotor of a MachineSCIM must be a " "LamSquirrelCage")
    if not isinstance(self.stator, LamSlotWind):
        raise M1C_WrongStator("The Stator of a MachineSCIM must be a " "LamSlotWind")
    if self.rotor.winding.p != self.stator.winding.p:
        raise M1C_SquirrelCagePError(
            "The stator and the rotor winding must " "have the same value for p"
        )
    if self.rotor.is_internal and (
        self.rotor.slot.comp_radius_mid_active() + self.rotor.Hscr / 2.0
        > self.stator.Rint
    ):
        raise M1C_RingTooLarge("The Ring is larger than the stator internal " "radius")


class M1C_WrongRotor(MachineCheckError):
    """ """

    pass


class M1C_WrongStator(MachineCheckError):
    """

    Parameters
    ----------

    Returns
    -------

    Raises
    ------
    with
        Winding

    """

    pass


class M1C_SquirrelCagePError(MachineCheckError):
    """

    Parameters
    ----------

    Returns
    -------

    Raises
    ------
    winding
        have a different p value as the rotor one

    """

    pass


class M1C_RingTooLarge(MachineCheckError):
    """ """

    pass



def has_magnet(self):
    """Return if any of the Holes have magnets

    Parameters
    ----------
    self : LamHole
        A LamHole object

    Returns
    -------
    has_magnet : bool
        True if any of the Holes have magnets
    """

    has_mag = [hole.has_magnet() for hole in self.hole]
    return any(has_mag)



# -*- coding: utf-8 -*-

import numpy as np


def get_point2cell(self, pt_indice):
    """Return all indices of cells containing a point.

    Parameters
    ----------
    self : CellMat
        an CellMat object
    pt_indice : int
        a point indice

    Returns
    -------
    pt_to_cell: ndarray
        Indices of cells containing the point

    """

    pt_to_cell = np.array([], dtype=int)
    connect = self.connectivity
    ind = self.indice

    if (
        connect is not None and len(connect.shape) > 1
    ):  # If there is more than 1 element
        Ielem = np.where(connect == pt_indice)[0]
        pt_to_cell = ind[Ielem]
    else:
        if sum(connect == pt_indice) > 0:
            pt_to_cell = ind

    return pt_to_cell



# -*- coding: utf-8 -*-


def comp_length_airgap_active(self):
    """Compute the airgap active length

    Parameters
    ----------
    self : Machine
        A Machine object

    Returns
    -------
    Lgap: float
        Airgap active length [m]

    """

    return min(self.rotor.L1, self.stator.L1)



# -*- coding: utf-8 -*-
from ....Classes.Segment import Segment
from ....Classes.Arc1 import Arc1


def build_geometry(self):
    """Compute the curve (Line) needed to plot the Slot.
    The ending point of a curve is the starting point of the next curve in
    the list

    Parameters
    ----------
    self : SlotW61
        A SlotW61 object

    Returns
    -------
    curve_list: list
        A list of 10 Segment

    """

    [Z1, Z2, Z3, Z4, Z5, Z6, Z7, Z8, Z9, Z10] = self._comp_point_coordinate()

    # Creation of curve
    curve_list = list()
    curve_list.append(Segment(Z1, Z2))
    curve_list.append(Segment(Z2, Z3))
    curve_list.append(Segment(Z3, Z4))
    curve_list.append(Segment(Z4, Z5))
    curve_list.append(Arc1(Z5, Z6, abs(Z5)))
    curve_list.append(Segment(Z6, Z7))
    curve_list.append(Segment(Z7, Z8))
    curve_list.append(Segment(Z8, Z9))
    curve_list.append(Segment(Z9, Z10))

    return curve_list



# -*- coding: utf-8 -*-
from numpy import exp
from ....Methods.Geometry.Circle import AngleRotationCircleError


def rotate(self, angle):
    """Rotation of the Circle of angle

    Parameters
    ----------
    self : Circle
        An Circle Object

    angle : float
        the angle of rotation [rad]

    Returns
    -------
    None

    Raises
    -------
    AngleRotationCircleError
        The angle must be a float or int
    """
    if not isinstance(angle, float) and not isinstance(angle, int):
        raise AngleRotationCircleError("The angle must be a float or int ")

    # check if Circle is correct"
    self.check()

    # Modification of the object from the rotation
    self.center = self.center * exp(1j * angle)
    if self.point_ref is not None:
        self.point_ref = self.point_ref * exp(1j * angle)



# -*- coding: utf-8 -*-

from numpy import arcsin, pi

from ....Methods.Slot.Slot import SlotCheckError
from ....Methods.Slot.HoleM50 import *


def check(self):
    """Check that the HoleM50 object is correct

    Parameters
    ----------
    self : HoleM50
        A HoleM50 object

    Returns
    -------
    None

    Raises
    _______
    S50_W01CheckError
        You must have W1 < W0
    S50_H23CheckError
        You must have H2 < H3
    S50_H01CheckError
        You must have H1 < H0
    S50_W5CheckError
        You must have W5 >=0
    S50_SpCheckError
        Slot pitch too small for the slot, reduce Zh, W3 or W0
    """
    Rext = self.get_Rext()

    # Check that everything is set
    if self.W0 is None:
        raise S50_NoneError("You must set W0 !")
    elif self.W1 is None:
        raise S50_NoneError("You must set W1 !")
    elif self.W2 is None:
        raise S50_NoneError("You must set W2 !")
    elif self.W3 is None:
        raise S50_NoneError("You must set W3 !")
    elif self.W4 is None:
        raise S50_NoneError("You must set W4 !")
    elif self.H0 is None:
        raise S50_NoneError("You must set H0 !")
    elif self.H1 is None:
        raise S50_NoneError("You must set H1 !")
    elif self.H2 is None:
        raise S50_NoneError("You must set H2 !")
    elif self.H3 is None:
        raise S50_NoneError("You must set H3 !")
    elif self.H4 is None:
        raise S50_NoneError("You must set H4 !")

    if self.W0 <= self.W1:
        raise S50_W01CheckError("You must have W1 < W0")

    if self.H3 <= self.H2:
        raise S50_H23CheckError("You must have H2 < H3")

    if self.H0 <= self.H1:
        raise S50_H01CheckError("You must have H1 < H0")

    if self.comp_W5() < 0:
        raise S50_W5CheckError("You must have W5 >=0")

    alpha_0 = 2 * arcsin(self.W0 / (2 * (Rext - self.H1)))  # W0 in rad
    alpha_3 = 2 * arcsin(self.W3 / (2 * (Rext - self.H1)))  # W3 in rad
    if alpha_0 + alpha_3 > 2 * pi / self.Zh:
        raise S50_SpCheckError("Slot pitch too small for the slot, reduce Zh, W3 or W0")



from numpy import array
from ....Functions.init_fig import init_fig
from ....Functions.Plot import TEXT_BOX


def plot(
    self,
    fig=None,
    ax=None,
    linestyle="solid",
    linewidth=1,
    color="black",
    label=None,
    offset_label=0,
    fontsize=12,
):
    """Plot the Arc

    Parameters
    ----------
    self : Arc
        An Arc object
    fig : Matplotlib.figure.Figure
        existing figure to use if None create a new one
    ax : Matplotlib.axes.Axes object
        Axis on which to plot the data
    linestyle : str
        Line of the line (solid, dotted...)
    linewidth : int
        Line Width
    color : str
        Color of the line
    label : str
        To add a label at the middle of the line
    offset_label : complex
        Complex value to shift the label from the middle
    fontsize : int
        Size of the font for the label (if any)
    """

    # Init fig
    (fig, axes, patch_leg, label_leg) = init_fig(fig=fig, ax=ax, shape="rectangle")

    # Plot the line
    points = self.discretize(nb_point=10000)
    axes.plot(
        points.real, points.imag, linestyle=linestyle, linewidth=linewidth, color=color
    )

    # Add the label (if needed)
    if label is not None:
        Zmid = self.get_middle()
        axes.text(
            Zmid.real + offset_label.real,
            Zmid.imag + offset_label.imag,
            label,
            fontsize=fontsize,
            bbox=TEXT_BOX,
        )



# -*- coding: utf-8 -*-


from PySide2.QtCore import Signal
from PySide2.QtWidgets import QMessageBox, QWidget

from ......GUI.Dialog.DMachineSetup.SMHoleMag.PHoleM50.PHoleM50 import PHoleM50
from ......GUI.Dialog.DMachineSetup.SMHoleMag.PHoleM51.PHoleM51 import PHoleM51
from ......GUI.Dialog.DMachineSetup.SMHoleMag.PHoleM52.PHoleM52 import PHoleM52
from ......GUI.Dialog.DMachineSetup.SMHoleMag.PHoleM53.PHoleM53 import PHoleM53
from ......GUI.Dialog.DMachineSetup.SMHoleMag.PHoleM54.PHoleM54 import PHoleM54
from ......GUI.Dialog.DMachineSetup.SMHoleMag.PHoleM57.PHoleM57 import PHoleM57
from ......GUI.Dialog.DMachineSetup.SMHoleMag.PHoleM58.PHoleM58 import PHoleM58
from ......GUI.Dialog.DMachineSetup.SMHoleMag.PHoleMUD.PHoleMUD import PHoleMUD
from ......GUI.Dialog.DMachineSetup.SMHoleMag.WHoleMag.Ui_WHoleMag import Ui_WHoleMag


class WHoleMag(Ui_WHoleMag, QWidget):
    """Widget to Setup a single Hole in a list"""

    # Signal to DMachineSetup to know that the save popup is needed
    saveNeeded = Signal()

    def __init__(self, parent, is_mag, index, matlib):
        """Initialize the GUI according to lamination

        Parameters
        ----------
        self : WHoleMag
            A WHoleMag object
        parent :
            A parent object containing the lamination (LamHole) to edit
        is_mag : bool
            False: no magnet in the Hole (for the SyRM)
        index : int
            Index of the hole to edit
        matlib : MatLib
            Material Library
        """

        # Build the interface according to the .ui file
        QWidget.__init__(self)
        self.setupUi(self)

        self.is_stator = False
        self.obj = parent.obj
        self.index = index
        self.is_mag = is_mag
        self.parent = parent
        self.matlib = matlib

        # Adapt the GUI to the current machine
        if is_mag:  # IPMSM
            self.wid_list = [
                PHoleM50,
                PHoleM51,
                PHoleM52,
                PHoleM53,
                PHoleM57,
                PHoleM58,
                PHoleMUD,
            ]
        else:  # SyRM
            self.wid_list = [
                PHoleM50,
                PHoleM51,
                PHoleM52,
                PHoleM53,
                PHoleM54,
                PHoleM57,
                PHoleM58,
                PHoleMUD,
            ]
        self.type_list = [wid.hole_type for wid in self.wid_list]
        self.name_list = [wid.hole_name for wid in self.wid_list]

        # Avoid erase all the parameters when navigating though the holes
        self.previous_hole = dict()
        for hole_type in self.type_list:
            self.previous_hole[hole_type] = None

        # Fill the combobox with the available hole
        self.c_hole_type.clear()
        for hole in self.name_list:
            self.c_hole_type.addItem(hole)
        self.c_hole_type.setCurrentIndex(
            self.type_list.index(type(self.obj.hole[index]))
        )

        # Regenerate the pages with the new values
        self.w_hole.setParent(None)
        self.w_hole = self.wid_list[self.c_hole_type.currentIndex()](
            hole=self.obj.hole[index], matlib=self.matlib
        )
        # Refresh the GUI
        self.main_layout.removeWidget(self.w_hole)
        self.main_layout.insertWidget(1, self.w_hole)

        # Connect the slot
        self.c_hole_type.currentIndexChanged.connect(self.set_hole_type)

    def emit_save(self):
        """Send a saveNeeded signal to the DMachineSetup"""
        self.saveNeeded.emit()

    def set_hole_type(self, c_index):
        """Initialize self.obj with the hole corresponding to index

        Parameters
        ----------
        self : WHoleMag
            A WHoleMag object
        c_index : int
            Index of the selected hole type in the combobox
        """

        # Save the hole
        hole = self.obj.hole[self.index]
        self.previous_hole[type(hole)] = hole

        # Call the corresponding constructor
        Zh = hole.Zh
        if self.previous_hole[self.type_list[c_index]] is None:
            # No previous hole of this type
            self.obj.hole[self.index] = self.type_list[c_index]()
            self.obj.hole[self.index]._set_None()  # No default value
            self.obj.hole[self.index].Zh = Zh
            if self.is_mag and self.obj.hole[self.index].has_magnet():  # IPMSM
                magnet = hole.get_magnet_by_id(0)
                self.obj.hole[self.index].set_magnet_by_id(0, magnet)
            elif self.obj.hole[self.index].has_magnet():  # SyRM
                self.obj.hole[self.index].remove_magnet()
        else:  # Load the previous hole of this type
            self.obj.hole[self.index] = self.previous_hole[self.type_list[c_index]]

        # Update the GUI
        self.w_hole.setParent(None)
        self.w_hole = self.wid_list[c_index](
            hole=self.obj.hole[self.index], matlib=self.matlib
        )
        self.w_hole.saveNeeded.connect(self.emit_save)
        # Refresh the GUI
        self.main_layout.removeWidget(self.w_hole)
        self.main_layout.insertWidget(1, self.w_hole)

        # Notify the machine GUI that the machine has changed
        self.saveNeeded.emit()

    def check(self):
        """Check that the current machine have all the needed field set

        Parameters
        ----------
        self : WHoleMag
            A WHoleMag widget

        Returns
        -------
        error : str
            Error message (return None if no error)
        """

        return self.w_hole.check()



# -*- coding: utf-8 -*-


def check(self):
    """Check that the Conductor object is correct

    Parameters
    ----------
    self : Conductor
        A Conductor object

    Returns
    -------
    None
    """
    pass



# -*- coding: utf-8 -*-

from numpy import array_equal, roll, squeeze, sum as np_sum
from numpy.linalg import norm


def comp_wind_periodicity(wind_mat):
    """Computes the winding pattern periodicity and symmetries

    Parameters
    ----------
    wind_mat : ndarray
        Matrix of the Winding

    Returns
    -------
    Nperw: int
        Number of electrical period of the winding

    """
    assert len(wind_mat.shape) == 4, "dim 4 expected for wind_mat"

    # Summing on all the layers (Nlay_r and Nlay_theta)
    wind_mat2 = squeeze(np_sum(np_sum(wind_mat, axis=1), axis=0))

    qs = wind_mat.shape[3]  # Number of phase
    Zs = wind_mat.shape[2]  # Number of Slot

    Nperw = 1  # Number of electrical period of the winding
    Nperslot = 1  # Periodicity of the winding in number of slots

    # Looking for the periodicity of each phase
    for q in range(0, qs):
        k = 1
        is_per = False
        while k <= Zs and not is_per:
            # We shift the array arround the slot and check if it's the same
            if array_equal(wind_mat2[:, q], roll(wind_mat2[:, q], shift=k)):
                is_per = True
            else:
                k += 1
        # least common multiple to find common periodicity between different phase
        Nperslot = lcm(Nperslot, k)

    # If Nperslot > Zs no symmetry
    if Nperslot > 0 and Nperslot < Zs:
        # nb of periods of the winding (2 means 180°)
        Nperw = Zs / float(Nperslot)
        # if Zs cannot be divided by Nperslot (non integer)
        if Nperw % 1 != 0:
            Nperw = 1

    # Check for anti symmetries in the elementary winding pattern
    if (
        Nperslot % 2 == 0
        and norm(
            wind_mat2[0 : Nperslot // 2, :] + wind_mat2[Nperslot // 2 : Nperslot, :]
        )
        == 0
    ):
        is_aper_wind = True
        Nperw = Nperw * 2
    else:
        is_aper_wind = False

    return int(Nperw), is_aper_wind


def gcd(a, b):
    """Return the greatest common divisor of a and b

    Parameters
    ----------
    a : int
        first number
    b : int
        second number

    Returns
    -------
    gcd : int
        greatest common divisor of a and b
    """
    while b:
        a, b = b, a % b
    return a


def lcm(a, b):
    """Return the least common multiple of a and b

    Parameters
    ----------
    a : int
        first number
    b : int
        second number

    Returns
    -------
    lcm : int
        least common multiple of a and b
    """
    return a * b // gcd(a, b)



# -*- coding: utf-8 -*-

import numpy as np
import copy

from pyleecan.Classes.CellMat import CellMat
from pyleecan.Classes.MeshMat import MeshMat
from pyleecan.Classes.PointMat import PointMat
from pyleecan.Classes.SolutionMat import SolutionMat
from pyleecan.definitions import PACKAGE_NAME


def get_group(self, group_names):
    """Return all attributes of a MeshSolution object with only the cells, points
    and corresponding solutions of the group. Solutions are converted as SolutionMat.

     Parameters
     ----------
     self : MeshSolution
         an MeshSolution object
     group_name : str
         the name of the group (e.g. "stator")

     Returns
     -------
     grp_cells: dict
         a dict sorted by cell type containing connectivity of the group

    """

    is_same_mesh = self.is_same_mesh
    dimension = self.dimension

    group_indices = list()
    label = ""
    is_interface = False

    # 1) get the indices of all targeted cell corresponding to group(s)
    sep_list = list()
    if isinstance(group_names, list):
        for grp in group_names:
            if grp == "/":
                # The groups before and after "/" are stored in different lists
                # to perform the interface.
                is_interface = True
                group_indices_init = group_indices.copy()
                group_indices = list()
                sep_list.append(group_indices_init)
            else:
                group_indices.extend(self.group[grp])
                label = label + grp + "_"
    elif isinstance(group_names, str):
        if group_names not in self.group:
            raise KeyError(
                group_names
                + " group doesn't exist (available groups: "
                + str(list(self.group.keys()))
                + ")"
            )
        group_indices.extend(self.group[group_names])
        label = label + group_names

    sep_list.append(group_indices)

    # 2) extract the corresponding connectivity and create a new mesh
    mesh_init = self.get_mesh()
    point_init = mesh_init.get_point()
    mesh_list = list()
    for sep in sep_list:
        connect_dict, nb_cell, indice_dict = mesh_init.get_cell(sep)

        node_indice = list()
        mesh_new = MeshMat()
        for key in connect_dict:
            node_indice.extend(np.unique(connect_dict[key]))
            mesh_new.cell[key] = CellMat(
                connectivity=connect_dict[key],
                nb_cell=len(connect_dict[key]),
                nb_pt_per_cell=mesh_init.cell[key].nb_pt_per_cell,
                indice=indice_dict[key],
                interpolation=mesh_init.cell[key].interpolation,
            )
        node_indice = np.unique(node_indice)

        mesh_new.point = PointMat(init_dict=mesh_init.point.as_dict())
        mesh_new.label = label

        mesh_list.append(mesh_new)

    # 3) if interface, create the corresponding new mesh (e.g. with triangle mesh,
    #    it creates only segment cells)
    if is_interface:
        mesh_interface = mesh_list[0].interface(mesh_list[1])
        connect_interface, nb_cell_interf, indices_interf = mesh_interface.get_cell()
        node_indice_interf = list()
        for key in connect_interface:
            node_indice_interf.extend(np.unique(connect_interface[key]))

        node_indice = np.unique(node_indice_interf)

    # 4) select the corresponding solutions
    sol_list = list()
    for sol in self.solution:
        type_cell_sol = sol.type_cell

        new_sol = None
        if type_cell_sol == "point":
            new_sol = sol.get_solution(indice=node_indice)
        elif not is_interface:  # Interface is only available for point solution.
            new_sol = sol.get_solution(indice=indice_dict[type_cell_sol])

        if new_sol is not None:
            sol_list.append(new_sol)

    # 5) Create the corresponding MeshSolution object
    if is_interface:
        mesh_interface.renum()
        mesh = mesh_interface
    else:
        mesh_new.renum()
        mesh = mesh_new

    meshsol_grp = self.copy()
    meshsol_grp.label = label
    meshsol_grp.mesh = [mesh]
    meshsol_grp.is_same_mesh = is_same_mesh
    meshsol_grp.solution = sol_list
    meshsol_grp.dimension = dimension

    return meshsol_grp



# -*- coding: utf-8 -*-

from numpy import array, nan, tile, newaxis, ones_like
from SciDataTool import DataTime, Data1D

from ....Classes.SolutionData import SolutionData
from ....Functions.getattr_recursive import getattr_recursive

from logging import getLogger


def comp_loss(self, output, part_label):
    """Compute the Losses"""
    # get logger
    logger = self.get_logger()

    # check inpurt
    if not "Stator" in part_label and not "Rotor" in part_label:
        logger.warning(
            f"LossModelWinding.comp_loss(): 'part_label'"
            + f" {part_label} not implemented yet."
        )
        return None, None

    # get the simulation and the lamination
    simu = output.simu
    lam = simu.machine.get_lam_by_label(part_label)

    # check that lamination has a winding
    if hasattr(lam, "winding") and lam.winding is not None:
        R = lam.comp_resistance_wind(T=self.temperature)
        if lam.is_stator:
            current = output.elec.get_Is()
        else:
            current = output.elec.get_Ir()

        axes_names = [axis.name for axis in current.axes]
        data_dict = current.get_along(*axes_names)

        data = DataTime(
            name=self.name,
            unit="W",
            symbol="Loss",
            axes=current.axes,
            values=R * data_dict[current.symbol] ** 2,
        )

        return data, None

    else:
        logger.warning("LossModelWinding.comp_loss(): Lamination has no winding.")
        return None, None



# -*- coding: utf-8 -*-
from numpy import abs as np_abs, angle as np_angle, arcsin, exp, pi, sqrt


def get_center(self):
    """Return the center of the arc

    Parameters
    ----------
    self : Arc1
        An Arc1 object

    Returns
    -------
    Zc: complex
        Complex coordinates of the center of the Arc1
    """

    self.check()

    # The center is on the bisection of [begin, end]
    z1 = self.begin
    z2 = self.end
    R = self.radius
    D12 = np_abs(z2 - z1)  # length of segment [begin,end]

    # Centre at the middle of begin and end (distance(Z1, Z2) = diameter )
    if np_abs(D12 - np_abs(2 * R)) < 1e-6:
        Zc = (z2 + z1) / 2.0
    else:
        # In the coordinate system begin on center and end on X > 0 axis
        if R > 0:  # Center is above the segment
            Zc = D12 / 2 + 1j * sqrt(R ** 2 - (D12 / 2) ** 2)
        else:
            Zc = D12 / 2 - 1j * sqrt(R ** 2 - (D12 / 2) ** 2)
        # Go back to the original coordinate system
        Zc = Zc * exp(1j * np_angle(z2 - z1)) + z1

    # Return (0,0) if the point is too close from 0
    if np_abs(Zc) < 1e-6:
        Zc = 0

    return Zc



# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/Simulation/EEC_PMSM.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/pyleecan/tree/master/pyleecan/Methods/Simulation/EEC_PMSM
"""

from os import linesep
from sys import getsizeof
from logging import getLogger
from ._check import check_var, raise_
from ..Functions.get_logger import get_logger
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from .EEC import EEC

# Import all class method
# Try/catch to remove unnecessary dependencies in unused method
try:
    from ..Methods.Simulation.EEC_PMSM.comp_parameters import comp_parameters
except ImportError as error:
    comp_parameters = error

try:
    from ..Methods.Simulation.EEC_PMSM.solve_EEC import solve_EEC
except ImportError as error:
    solve_EEC = error

try:
    from ..Methods.Simulation.EEC_PMSM.gen_drive import gen_drive
except ImportError as error:
    gen_drive = error

try:
    from ..Methods.Simulation.EEC_PMSM.comp_joule_losses import comp_joule_losses
except ImportError as error:
    comp_joule_losses = error


from ._check import InitUnKnowClassError
from .IndMag import IndMag
from .FluxLink import FluxLink
from .Drive import Drive


class EEC_PMSM(EEC):
    """Electric module: Electrical Equivalent Circuit"""

    VERSION = 1

    # Check ImportError to remove unnecessary dependencies in unused method
    # cf Methods.Simulation.EEC_PMSM.comp_parameters
    if isinstance(comp_parameters, ImportError):
        comp_parameters = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use EEC_PMSM method comp_parameters: " + str(comp_parameters)
                )
            )
        )
    else:
        comp_parameters = comp_parameters
    # cf Methods.Simulation.EEC_PMSM.solve_EEC
    if isinstance(solve_EEC, ImportError):
        solve_EEC = property(
            fget=lambda x: raise_(
                ImportError("Can't use EEC_PMSM method solve_EEC: " + str(solve_EEC))
            )
        )
    else:
        solve_EEC = solve_EEC
    # cf Methods.Simulation.EEC_PMSM.gen_drive
    if isinstance(gen_drive, ImportError):
        gen_drive = property(
            fget=lambda x: raise_(
                ImportError("Can't use EEC_PMSM method gen_drive: " + str(gen_drive))
            )
        )
    else:
        gen_drive = gen_drive
    # cf Methods.Simulation.EEC_PMSM.comp_joule_losses
    if isinstance(comp_joule_losses, ImportError):
        comp_joule_losses = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use EEC_PMSM method comp_joule_losses: "
                    + str(comp_joule_losses)
                )
            )
        )
    else:
        comp_joule_losses = comp_joule_losses
    # save and copy methods are available in all object
    save = save
    copy = copy
    # get_logger method is available in all object
    get_logger = get_logger

    def __init__(
        self,
        indmag=None,
        fluxlink=None,
        parameters=-1,
        freq0=None,
        drive=None,
        init_dict=None,
        init_str=None,
    ):
        """Constructor of the class. Can be use in three ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for pyleecan type, -1 will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary with property names as keys
        - __init__ (init_str = s) s must be a string
        s is the file path to load

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for pyleecan Object"""

        if init_str is not None:  # Load from a file
            init_dict = load_init_dict(init_str)[1]
        if init_dict is not None:  # Initialisation by dict
            assert type(init_dict) is dict
            # Overwrite default value with init_dict content
            if "indmag" in list(init_dict.keys()):
                indmag = init_dict["indmag"]
            if "fluxlink" in list(init_dict.keys()):
                fluxlink = init_dict["fluxlink"]
            if "parameters" in list(init_dict.keys()):
                parameters = init_dict["parameters"]
            if "freq0" in list(init_dict.keys()):
                freq0 = init_dict["freq0"]
            if "drive" in list(init_dict.keys()):
                drive = init_dict["drive"]
        # Set the properties (value check and convertion are done in setter)
        self.indmag = indmag
        self.fluxlink = fluxlink
        self.parameters = parameters
        self.freq0 = freq0
        self.drive = drive
        # Call EEC init
        super(EEC_PMSM, self).__init__()
        # The class is frozen (in EEC init), for now it's impossible to
        # add new properties

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        EEC_PMSM_str = ""
        # Get the properties inherited from EEC
        EEC_PMSM_str += super(EEC_PMSM, self).__str__()
        if self.indmag is not None:
            tmp = self.indmag.__str__().replace(linesep, linesep + "\t").rstrip("\t")
            EEC_PMSM_str += "indmag = " + tmp
        else:
            EEC_PMSM_str += "indmag = None" + linesep + linesep
        if self.fluxlink is not None:
            tmp = self.fluxlink.__str__().replace(linesep, linesep + "\t").rstrip("\t")
            EEC_PMSM_str += "fluxlink = " + tmp
        else:
            EEC_PMSM_str += "fluxlink = None" + linesep + linesep
        EEC_PMSM_str += "parameters = " + str(self.parameters) + linesep
        EEC_PMSM_str += "freq0 = " + str(self.freq0) + linesep
        if self.drive is not None:
            tmp = self.drive.__str__().replace(linesep, linesep + "\t").rstrip("\t")
            EEC_PMSM_str += "drive = " + tmp
        else:
            EEC_PMSM_str += "drive = None" + linesep + linesep
        return EEC_PMSM_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False

        # Check the properties inherited from EEC
        if not super(EEC_PMSM, self).__eq__(other):
            return False
        if other.indmag != self.indmag:
            return False
        if other.fluxlink != self.fluxlink:
            return False
        if other.parameters != self.parameters:
            return False
        if other.freq0 != self.freq0:
            return False
        if other.drive != self.drive:
            return False
        return True

    def compare(self, other, name="self"):
        """Compare two objects and return list of differences"""

        if type(other) != type(self):
            return ["type(" + name + ")"]
        diff_list = list()

        # Check the properties inherited from EEC
        diff_list.extend(super(EEC_PMSM, self).compare(other, name=name))
        if (other.indmag is None and self.indmag is not None) or (
            other.indmag is not None and self.indmag is None
        ):
            diff_list.append(name + ".indmag None mismatch")
        elif self.indmag is not None:
            diff_list.extend(self.indmag.compare(other.indmag, name=name + ".indmag"))
        if (other.fluxlink is None and self.fluxlink is not None) or (
            other.fluxlink is not None and self.fluxlink is None
        ):
            diff_list.append(name + ".fluxlink None mismatch")
        elif self.fluxlink is not None:
            diff_list.extend(
                self.fluxlink.compare(other.fluxlink, name=name + ".fluxlink")
            )
        if other._parameters != self._parameters:
            diff_list.append(name + ".parameters")
        if other._freq0 != self._freq0:
            diff_list.append(name + ".freq0")
        if (other.drive is None and self.drive is not None) or (
            other.drive is not None and self.drive is None
        ):
            diff_list.append(name + ".drive None mismatch")
        elif self.drive is not None:
            diff_list.extend(self.drive.compare(other.drive, name=name + ".drive"))
        return diff_list

    def __sizeof__(self):
        """Return the size in memory of the object (including all subobject)"""

        S = 0  # Full size of the object

        # Get size of the properties inherited from EEC
        S += super(EEC_PMSM, self).__sizeof__()
        S += getsizeof(self.indmag)
        S += getsizeof(self.fluxlink)
        if self.parameters is not None:
            for key, value in self.parameters.items():
                S += getsizeof(value) + getsizeof(key)
        S += getsizeof(self.freq0)
        S += getsizeof(self.drive)
        return S

    def as_dict(self, **kwargs):
        """
        Convert this object in a json serializable dict (can be use in __init__).
        Optional keyword input parameter is for internal use only
        and may prevent json serializability.
        """

        # Get the properties inherited from EEC
        EEC_PMSM_dict = super(EEC_PMSM, self).as_dict(**kwargs)
        if self.indmag is None:
            EEC_PMSM_dict["indmag"] = None
        else:
            EEC_PMSM_dict["indmag"] = self.indmag.as_dict(**kwargs)
        if self.fluxlink is None:
            EEC_PMSM_dict["fluxlink"] = None
        else:
            EEC_PMSM_dict["fluxlink"] = self.fluxlink.as_dict(**kwargs)
        EEC_PMSM_dict["parameters"] = (
            self.parameters.copy() if self.parameters is not None else None
        )
        EEC_PMSM_dict["freq0"] = self.freq0
        if self.drive is None:
            EEC_PMSM_dict["drive"] = None
        else:
            EEC_PMSM_dict["drive"] = self.drive.as_dict(**kwargs)
        # The class name is added to the dict for deserialisation purpose
        # Overwrite the mother class name
        EEC_PMSM_dict["__class__"] = "EEC_PMSM"
        return EEC_PMSM_dict

    def _set_None(self):
        """Set all the properties to None (except pyleecan object)"""

        if self.indmag is not None:
            self.indmag._set_None()
        if self.fluxlink is not None:
            self.fluxlink._set_None()
        self.parameters = None
        self.freq0 = None
        if self.drive is not None:
            self.drive._set_None()
        # Set to None the properties inherited from EEC
        super(EEC_PMSM, self)._set_None()

    def _get_indmag(self):
        """getter of indmag"""
        return self._indmag

    def _set_indmag(self, value):
        """setter of indmag"""
        if isinstance(value, str):  # Load from file
            value = load_init_dict(value)[1]
        if isinstance(value, dict) and "__class__" in value:
            class_obj = import_class(
                "pyleecan.Classes", value.get("__class__"), "indmag"
            )
            value = class_obj(init_dict=value)
        elif type(value) is int and value == -1:  # Default constructor
            value = IndMag()
        check_var("indmag", value, "IndMag")
        self._indmag = value

        if self._indmag is not None:
            self._indmag.parent = self

    indmag = property(
        fget=_get_indmag,
        fset=_set_indmag,
        doc=u"""Magnetic inductance

        :Type: IndMag
        """,
    )

    def _get_fluxlink(self):
        """getter of fluxlink"""
        return self._fluxlink

    def _set_fluxlink(self, value):
        """setter of fluxlink"""
        if isinstance(value, str):  # Load from file
            value = load_init_dict(value)[1]
        if isinstance(value, dict) and "__class__" in value:
            class_obj = import_class(
                "pyleecan.Classes", value.get("__class__"), "fluxlink"
            )
            value = class_obj(init_dict=value)
        elif type(value) is int and value == -1:  # Default constructor
            value = FluxLink()
        check_var("fluxlink", value, "FluxLink")
        self._fluxlink = value

        if self._fluxlink is not None:
            self._fluxlink.parent = self

    fluxlink = property(
        fget=_get_fluxlink,
        fset=_set_fluxlink,
        doc=u"""Flux Linkage

        :Type: FluxLink
        """,
    )

    def _get_parameters(self):
        """getter of parameters"""
        return self._parameters

    def _set_parameters(self, value):
        """setter of parameters"""
        if type(value) is int and value == -1:
            value = dict()
        check_var("parameters", value, "dict")
        self._parameters = value

    parameters = property(
        fget=_get_parameters,
        fset=_set_parameters,
        doc=u"""Parameters of the EEC: computed if empty, or enforced

        :Type: dict
        """,
    )

    def _get_freq0(self):
        """getter of freq0"""
        return self._freq0

    def _set_freq0(self, value):
        """setter of freq0"""
        check_var("freq0", value, "float")
        self._freq0 = value

    freq0 = property(
        fget=_get_freq0,
        fset=_set_freq0,
        doc=u"""Frequency

        :Type: float
        """,
    )

    def _get_drive(self):
        """getter of drive"""
        return self._drive

    def _set_drive(self, value):
        """setter of drive"""
        if isinstance(value, str):  # Load from file
            value = load_init_dict(value)[1]
        if isinstance(value, dict) and "__class__" in value:
            class_obj = import_class(
                "pyleecan.Classes", value.get("__class__"), "drive"
            )
            value = class_obj(init_dict=value)
        elif type(value) is int and value == -1:  # Default constructor
            value = Drive()
        check_var("drive", value, "Drive")
        self._drive = value

        if self._drive is not None:
            self._drive.parent = self

    drive = property(
        fget=_get_drive,
        fset=_set_drive,
        doc=u"""Drive

        :Type: Drive
        """,
    )



from numpy import arcsin, exp, pi, sqrt

from ....Methods.Slot.SlotW16 import S16OutterError
from ....Classes.Arc1 import Arc1


def _comp_point_coordinate(self):
    """Compute the point coordinates needed to plot the Slot.

    Parameters
    ----------
    self : SlotW16
        A SlotW16 object

    Returns
    -------
    point_dict: dict
        A dict of the slot point coordinates
    """

    Rbo = self.get_Rbo()

    hsp = pi / self.Zs  # Half slot pitch

    if self.is_outwards():
        raise S16OutterError("Slot Type 16 can't be used on inner lamination")

    # ZXt => Complex coordinate in the tooth ref
    Z1 = Rbo * exp(-1j * self.W0 / 2)
    Z2 = (Rbo - self.H0) * exp(-1j * self.W0 / 2)

    # Tooth angular width
    alphaT = 2 * arcsin(self.W3 * 0.5 / (Rbo - self.H0 - self.H2))
    Z5 = (Rbo - self.H0 - self.H2) * exp(-1j * (hsp - alphaT / 2))
    #  In the ref: O-tooth as axis
    # Z3 on cercle O => (x3t**2+y3t**2) = (Rbo-H0)**2
    # Zc on cercle O => (xct**2+yct**2) = (Rbo-H0-R1)**2
    # Z3 on cercle Zc => (xct-x3t)**2+(yct-y3t)**2 = R1**2
    # y4t = y5t = yct - R1
    # x4t = xct
    # O, Z3, Zc align => -y3t*xct+x3t*yct = 0
    Z5t = Z5 * exp(1j * hsp)
    y5t = Z5t.imag
    xct = sqrt(
        self.H0 ** 2
        + 2 * self.H0 * self.R1
        - 2 * self.H0 * Rbo
        - 2 * self.R1 * Rbo
        - 2 * self.R1 * y5t
        + Rbo ** 2
        - y5t ** 2
    )
    x3t = (self.H0 - Rbo) * xct / (self.H0 + self.R1 - Rbo)
    x4t = sqrt((self.H0 - Rbo - y5t) * (self.H0 + 2 * self.R1 - Rbo + y5t))
    y3t = (self.H0 - Rbo) * (self.R1 + y5t) / (self.H0 + self.R1 - Rbo)
    y4t = y5t
    Z4t = x4t + 1j * y4t
    Z3t = x3t + 1j * y3t

    Z4 = Z4t * exp(-1j * hsp)
    Z3 = Z3t * exp(-1j * hsp)

    point_dict = dict()
    # symetry
    point_dict["Z1"] = Z1
    point_dict["Z2"] = Z2
    point_dict["Z3"] = Z3
    point_dict["Z4"] = Z4
    point_dict["Z5"] = Z5
    point_dict["Z6"] = Z5.conjugate()
    point_dict["Z7"] = Z4.conjugate()
    point_dict["Z8"] = Z3.conjugate()
    point_dict["Z9"] = Z2.conjugate()
    point_dict["Z10"] = Z1.conjugate()
    # Compute center
    A = Arc1(Z3, Z4, -self.R1, is_trigo_direction=False)
    point_dict["Zc1"] = A.get_center()
    point_dict["Zc2"] = point_dict["Zc1"].conjugate()
    return point_dict



# -*- coding: utf-8 -*-

from matplotlib.patches import Circle, Patch
from matplotlib.pyplot import axis, legend
from numpy import exp, pi

from ....Functions.init_fig import init_fig


def plot(self, fig=None):
    """Plot the Ventilation in a matplotlib fig

    Parameters
    ----------
    self : VentilationCirc
        A VentilationCirc object
    fig :
        if None, open a new fig and plot, else add to the current
        one (Default value = None)

    Returns
    -------
    None
    """
    self.check()

    patches = list()

    if fig is None:
        color = "k"  # Only the vents are plot
    else:
        color = "w"  # Vents are "air" on a Lamination

    for ii in range(self.Zh.size):  # For every ventilation group
        for jj in range(int(self.Zh[ii])):
            # For every ventilation in one group
            # Computation of center coordinates
            Zc = self.H0[ii] * exp(1j * (jj * 2 * pi / self.Zh[ii] + self.Alpha0[ii]))
            # Creation of the Circle
            patches.append(Circle((Zc.real, Zc.imag), self.D0[ii] / 2.0, color=color))

    # Display the result
    (fig, axes, patch_leg, label_leg) = init_fig(fig)
    axes.set_xlabel("(m)")
    axes.set_ylabel("(m)")
    axes.set_title("Axial Ventilation Ducts")

    # Add the magnet to the fig
    for patch in patches:
        axes.add_patch(patch)

    # Axis Setup
    axis("equal")
    Lim = self.comp_radius()[1] * 1.2
    axes.set_xlim(-Lim, Lim)
    axes.set_ylim(-Lim, Lim)

    # Legend setup
    if color != "w":
        patch_leg.append(Patch(color="k"))
        label_leg.append("Ventilations")

        legend(patch_leg, label_leg)
    fig.show()



from SciDataTool.Functions import AxisError


def create_from_axis(axis_in, per, is_aper, is_include_per, is_remove_aper=False):
    """
    Create axis input axis accounting for (anti-)periodicity changes imposed
    by physics and model inputs

    Parameters
    ----------
    axis_in : Data
        The input axis coming from previous output (a Data object such as Data1D or DataLinspace)
    per: int
        machine periodicity along current axis
    is_aper: bool
        True if the machine is anti-periodic along current axis
    is_include_per: bool
        True if the model requires to include periodicity
    is_remove_aper: bool
        True if the model requires to remove anti-periodicity

    Returns
    -------
    axis_out : Data
        The output axis (a Data object such as Data1D or DataLinspace)
    is_include_per : bool
        Returns is_include_per in case periodicity is activated in the model but cannot be applied

    """

    # Getting the computation axes (with or without periodicity)
    if is_include_per:
        try:
            # Reduce axis to the machine periodicity
            per = per * 2 if is_aper else per
            axis_out = axis_in.get_axis_periodic(per, is_aper and not is_remove_aper)

        except AxisError:
            # Periodicity cannot be applied, return full axis
            axis_out = axis_in.copy()
            is_include_per = False

    else:
        # Return full axis
        axis_out = axis_in.copy()

    return axis_out, is_include_per



# -*- coding: utf-8 -*-


def comp_mass_magnet_id(self, index):
    """Compute the mass of the hole magnet of the corresponding index

    Parameters
    ----------
    self : HoleMag
        A HoleMag object
    index : int
        Index of the magnet to compute the surface

    Returns
    -------
    Mmag: float
        Mass of the Magnet [m**2]
    """

    mag_list = self.get_magnet_list()
    if mag_list[index] is None:
        return 0
    else:
        return self.comp_surface_magnet_id(index) * mag_list[index].Lmag



# -*- coding: utf-8 -*-

from ....Classes.Lamination import Lamination


def comp_output_geo(self):
    """Compute the main geometry output

    Parameters
    ----------
    self : Lamination
        A Lamination object

    Returns
    -------
    output: OutGeoLam
        Main geometry output of the lamintion

    """

    output = Lamination.comp_output_geo(self)
    output.Ksfill = self.comp_fill_factor()
    if self.slot is None:
        output.S_slot = 0
        output.S_slot_wind = 0
    else:
        output.S_slot = self.slot.comp_surface()
        output.S_slot_wind = self.slot.comp_surface_active()
    # output.S_wind_act = self.winding.conductor.comp_surface_active()

    return output



# -*- coding: utf-8 -*-


def comp_surface(self):
    """Compute the Slot total surface (by analytical computation).
    Caution, the bottom of the Slot is an Arc

    Parameters
    ----------
    self : SlotW24
        A SlotW24 object

    Returns
    -------
    S: float
        Slot total surface [m**2]

    """

    return self.comp_surface_active()



# -*- coding: utf-8 -*-

from numpy import pi


def comp_surface(self):
    """Compute the surface of all the axial ventilation ducts

    Parameters
    ----------
    self : VentilationPolar
        A VentilationPolar object

    Returns
    -------
    surface: float
        Axial ventilation ducts total surface [m**2]

    """

    self.check()

    return (
        pi * ((self.H0 + self.D0) ** 2 - self.H0 ** 2) / (2 * pi / (self.Zh * self.W1))
    )



# -*- coding: utf-8 -*-

from numpy import arcsin, cos, exp, angle, pi, sin, tan, array

from pyleecan.Classes.Segment import Segment
from pyleecan.Classes.SurfLine import SurfLine
from pyleecan.Functions.Geometry.inter_line_line import inter_line_line
import matplotlib.pyplot as plt


def build_geometry(self, alpha=0, delta=0, is_simplified=False):
    """Compute the curve (Segment) needed to plot the Slot.
    The ending point of a curve is the starting point of the next curve in
    the list

    Parameters
    ----------
    self : HoleM57
        A HoleM57 object
    alpha : float
        Angle to rotate the slot (Default value = 0) [rad]
    delta : complex
        Complex to translate the slot (Default value = 0)
    is_simplified : bool
       True to avoid line superposition

    Returns
    -------
    surf_list: list
        List of SurfLine needed to draw the HoleM57

    """

    if self.get_is_stator():  # check if the slot is on the stator
        st = "_Stator"
    else:
        st = "_Rotor"

    # Defining type of magnetization of magnet 0 and magnet 1
    if self.magnet_0:
        if self.magnet_0.type_magnetization == 0:
            type_mag_0 = "_Radial"
        else:
            type_mag_0 = "_Parallel"
    else:
        type_mag_0 = "None"

    if self.magnet_1:
        if self.magnet_1.type_magnetization == 0:
            type_mag_1 = "_Radial"
        else:
            type_mag_1 = "_Parallel"
    else:
        type_mag_1 = "None"


    # Get all the points
    point_dict = self._comp_point_coordinate()
    Z1 = point_dict["Z1"]
    Z2 = point_dict["Z2"]
    Z3 = point_dict["Z3"]
    Z4 = point_dict["Z4"]
    Z5 = point_dict["Z5"]
    Z6 = point_dict["Z6"]
    Z7 = point_dict["Z7"]
    Z8 = point_dict["Z8"]

    Z1s = point_dict["Z1s"]
    Z2s = point_dict["Z2s"]
    Z3s = point_dict["Z3s"]
    Z4s = point_dict["Z4s"]
    Z5s = point_dict["Z5s"]
    Z6s = point_dict["Z6s"]
    Z7s = point_dict["Z7s"]
    Z8s = point_dict["Z8s"]

    surf_list = list()
    # Z_list = array([Z1, Z2, Z3, Z4, Z5, Z6, Z7, Z8])
    # plt.plot(Z_list.real, Z_list.imag, "x r")
    # for ii in range(8):
    #     plt.text(Z_list[ii].real, Z_list[ii].imag, "Z" + str(ii + 1))
    # plt.show()
    # Check schematics:
    assert abs(abs(Z7 - Z8) - self.W2) < 1e-6
    assert abs(abs(Z6 - Z7) - self.W4) < 1e-6
    assert abs(abs(Z2 - Z3) - self.W4) < 1e-6
    assert abs(abs(Z2 - Z7) - self.H2) < 1e-6
    assert abs(abs(Z4 - Z4s) - self.W1) < 1e-6
    assert abs(abs(Z5 - Z5s) - self.W1) < 1e-6
    # TODO: Create all the surfaces for all the cases
    # (with/without magnet W1>0 or W1=0)
    # Air surface (W3) with magnet_0
    curve_list = list()
    curve_list.append(Segment(Z1, Z2))
    curve_list.append(Segment(Z2, Z7))
    if self.W2 > 0:
        curve_list.append(Segment(Z7, Z8))
    curve_list.append(Segment(Z8, Z1))

    # initiating the label of the line on the air surface
    # curve_list = set_name_line(curve_list, "hole_1_line")
    point_ref = (Z1 + Z2 + Z7 + Z8) / 4
    S1 = SurfLine(line_list=curve_list, label="Hole" + st, point_ref=point_ref)

    # Magnet_0 surface
    curve_list = list()
    if is_simplified:
        curve_list.append(Segment(Z7, Z2))
        curve_list.append(Segment(Z3, Z6))
    else:
        curve_list.append(Segment(Z2, Z3))
        curve_list.append(Segment(Z3, Z6))
        curve_list.append(Segment(Z6, Z7))
        curve_list.append(Segment(Z7, Z2))

    # initiating the label of the line of the magnet surface
    # curve_list = set_name_line(curve_list, "magnet_1_line")
    point_ref = (Z2 + Z3 + Z6 + Z7) / 4
    S2 = SurfLine(
        line_list=curve_list,
        label="HoleMagnet" + st + type_mag_0 + "_N_R0_T0_S0",
        point_ref=point_ref,
    )

    # Air surface with magnet_0 and W1 > 0
    curve_list = list()
    curve_list.append(Segment(Z4, Z5))
    curve_list.append(Segment(Z5, Z6))
    curve_list.append(Segment(Z6, Z3))
    if abs(Z4 - Z3) > 1e-6:
        curve_list.append(Segment(Z3, Z4))

    # initiating the label of the line on the air surface
    # curve_list = set_name_line(curve_list, "hole_2_line")
    point_ref = (Z6 + Z5 + Z4) / 3

    S3 = SurfLine(line_list=curve_list, label="Hole" + st, point_ref=point_ref)

    # Symmetry Air surface (W3) with magnet_1
    curve_list = list()
    curve_list.append(Segment(Z2s, Z1s))
    curve_list.append(Segment(Z1s, Z8s))
    if self.W2 > 0:
        curve_list.append(Segment(Z8s, Z7s))
    curve_list.append(Segment(Z7s, Z2s))
    point_ref = (Z1s + Z2s + Z8s + Z7s) / 4
    # initiating the label of the line on the air surface
    # curve_list = set_name_line(curve_list, "hole_3_line")
    S4 = SurfLine(line_list=curve_list, label="Hole" + st, point_ref=point_ref)

    # magnet_1 surface
    curve_list = list()
    if is_simplified:
        curve_list.append(Segment(Z2s, Z7s))
        curve_list.append(Segment(Z6s, Z3s))
    else:
        curve_list.append(Segment(Z3s, Z2s))
        curve_list.append(Segment(Z2s, Z7s))
        curve_list.append(Segment(Z7s, Z6s))
        curve_list.append(Segment(Z6s, Z3s))
    point_ref = (Z3s + Z2s + Z7s + Z6s) / 4

    # initiating the label of the line on the magnet surface
    # curve_list = set_name_line(curve_list, "magnet_2_line")
    S5 = SurfLine(
        line_list=curve_list,
        label="HoleMagnet" + st + type_mag_1 + "_N_R0_T1_S0",
        point_ref=point_ref,
    )

    # Air surface with magnet_1 and W1 > 0
    curve_list = list()
    curve_list.append(Segment(Z3s, Z6s))
    curve_list.append(Segment(Z6s, Z5s))
    curve_list.append(Segment(Z5s, Z4s))
    if abs(Z4 - Z3) > 1e-6:
        curve_list.append(Segment(Z4s, Z3s))

    point_ref = (Z3s + Z6s + Z5s) / 3

    # initiating the label of the line on the air surface
    # curve_list = set_name_line(curve_list, "hole_4_line")
    S6 = SurfLine(line_list=curve_list, label="Hole" + st, point_ref=point_ref)

    # Air surface between magnet_0 and magnet_1 with W1 == 0
    curve_list = list()
    curve_list.append(Segment(Z3, Z4))
    curve_list.append(Segment(Z4, Z3s))
    curve_list.append(Segment(Z3s, Z6s))
    curve_list.append(Segment(Z6s, Z5))
    curve_list.append(Segment(Z5, Z6))
    curve_list.append(Segment(Z6, Z3))

    # initiating the label of the line on the air surface
    # curve_list = set_name_line(curve_list, "hole_2_line")
    point_ref = (Z5 + Z4) / 2

    S7 = SurfLine(line_list=curve_list, label="Hole" + st, point_ref=point_ref)

    # Air surface without magnet_0 and W1 > 0
    curve_list = list()
    curve_list.append(Segment(Z1, Z8))
    curve_list.append(Segment(Z8, Z5))
    curve_list.append(Segment(Z5, Z4))
    curve_list.append(Segment(Z4, Z1))

    # initiating the label of the line on the air surface
    # curve_list = set_name_line(curve_list, "hole_1_line")
    point_ref = (Z5 + Z1) / 2

    S8 = SurfLine(line_list=curve_list, label="Hole" + st, point_ref=point_ref)

    # Air surface without magnet_1 and W1 > 0
    curve_list = list()
    curve_list.append(Segment(Z1s, Z8s))
    curve_list.append(Segment(Z8s, Z5s))
    curve_list.append(Segment(Z5s, Z4s))
    curve_list.append(Segment(Z4s, Z1s))

    # initiating the label of the line on the air surface
    # curve_list = set_name_line(curve_list, "hole_2_line")
    point_ref = (Z5s + Z1s) / 2

    S9 = SurfLine(line_list=curve_list, label="Hole" + st, point_ref=point_ref)

    # Air surface No magnet and W1 == 0
    curve_list = list()
    curve_list.append(Segment(Z4, Z1))
    curve_list.append(Segment(Z1, Z8))
    curve_list.append(Segment(Z8, Z5))
    curve_list.append(Segment(Z5, Z8s))
    curve_list.append(Segment(Z8s, Z1s))
    curve_list.append(Segment(Z1s, Z4))

    # initiating the label of the line on the air surface
    # curve_list = set_name_line(curve_list, "hole_1_line")
    point_ref = (Z4 + Z5) / 2

    S12 = SurfLine(line_list=curve_list, label="Hole" + st, point_ref=point_ref)

    # Create the surface list by selecting the correct ones
    if self.magnet_0 and self.magnet_1 and self.W1 > 0:
        surf_list = [S1, S2, S3, S6, S5, S4]
    elif self.magnet_0 and self.magnet_1 and self.W1 == 0:
        surf_list = [S1, S2, S7, S5, S4]
    # elif self.magnet_0 and not self.magnet_1 and self.W1 > 0:
    #     surf_list = [S1, S2, S3, S9]
    # elif self.magnet_0 and not self.magnet_1 and self.W1 == 0:
    #     surf_list = [S1, S2, S10]
    # elif not self.magnet_0 and self.magnet_1 and self.W1 > 0:
    #     surf_list = [S8, S6, S5, S4]
    # elif not self.magnet_0 and self.magnet_1 and self.W1 == 0:
    #     surf_list = [S11, S5, S4]
    elif not self.magnet_0 and not self.magnet_1 and self.W1 > 0:
        surf_list = [S8, S9]
    elif not self.magnet_0 and not self.magnet_1 and self.W1 == 0:
        surf_list = [S12]
    else:
        raise Exception("Not implemented Yet")

    # Apply the transformations
    for surf in surf_list:
        surf.rotate(alpha)
        surf.translate(delta)

    return surf_list


def set_name_line(hole_lines, name):
    """Define the  label of each line of the hole

    Parameters
    ----------
    hole_lines: list
        a list of line object of the slot
    name: str
        the name to give to the line
    Returns
    -------
    hole_lines: list
        List of line object with label
    """

    for ii in range(len(hole_lines)):
        hole_lines[ii].label = name + "_" + str(ii)
    return hole_lines




# -*- coding: utf-8 -*-

import numpy as np


def shape_function(self, g_point, nb_gpt):
    """Return the values of linear shape functions in reference 2 node segment for a given point

    Parameters
     ----------
    self : RefSegmentP1
         an RefCell object
    point : ndarray
        ref point

     Returns
     -------
     value: array
         interpolated field

    """

    nb_func = 2
    values = np.zeros(
        (nb_gpt, 1, nb_func), dtype=float
    )  # the "1" dimension is important for scalar product calculations

    for ig in range(nb_gpt):
        if nb_gpt == 1:
            [x, y] = g_point
        else:
            [x, y] = g_point[ig, :]

        if (x >= -1) and (x <= 1):
            values[ig, 0, 0] = (1 - x) / 2
            values[ig, 0, 1] = (1 + x) / 2
        elif x < -1:
            values[ig, 0, 0] = (3 + x) / 2
            values[ig, 0, 1] = -(1 + x) / 2
        elif x > 1:
            values[ig, 0, 0] = -(1 - x) / 2
            values[ig, 0, 1] = (3 - x) / 2
    return values, nb_func



# -*- coding: utf-8 -*-

from ....Functions.Structural.conversions import pol2cart


def get_shape_xyz(self):
    """Return the mode shape in cartesian coordinates.

    Parameters
    ----------
    self : Mode
        a Mode object

    Returns
    -------
    shape : ndarray
        ndarray of the shape (Nnodes*Ndof)
    """

    if self.shape_xyz is not None and self.shape_xyz.size != 0:
        return self.shape_xyz
    else:
        shape_pol = self.shape_pol
        points = self.parent.mesh.get_points()
        return pol2cart(shape_pol, points)



# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/Machine/WindingUD.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/pyleecan/tree/master/pyleecan/Methods/Machine/WindingUD
"""

from os import linesep
from sys import getsizeof
from logging import getLogger
from ._check import set_array, check_var, raise_
from ..Functions.get_logger import get_logger
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from .Winding import Winding

# Import all class method
# Try/catch to remove unnecessary dependencies in unused method
try:
    from ..Methods.Machine.WindingUD.comp_connection_mat import comp_connection_mat
except ImportError as error:
    comp_connection_mat = error

try:
    from ..Methods.Machine.WindingUD.get_dim_wind import get_dim_wind
except ImportError as error:
    get_dim_wind = error


from numpy import array, array_equal
from ._check import InitUnKnowClassError
from .Conductor import Conductor


class WindingUD(Winding):
    """User defined winding"""

    VERSION = 1
    NAME = "User defined"

    # Check ImportError to remove unnecessary dependencies in unused method
    # cf Methods.Machine.WindingUD.comp_connection_mat
    if isinstance(comp_connection_mat, ImportError):
        comp_connection_mat = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use WindingUD method comp_connection_mat: "
                    + str(comp_connection_mat)
                )
            )
        )
    else:
        comp_connection_mat = comp_connection_mat
    # cf Methods.Machine.WindingUD.get_dim_wind
    if isinstance(get_dim_wind, ImportError):
        get_dim_wind = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use WindingUD method get_dim_wind: " + str(get_dim_wind)
                )
            )
        )
    else:
        get_dim_wind = get_dim_wind
    # save and copy methods are available in all object
    save = save
    copy = copy
    # get_logger method is available in all object
    get_logger = get_logger

    def __init__(
        self,
        user_wind_mat=None,
        is_reverse_wind=False,
        Nslot_shift_wind=0,
        qs=3,
        Ntcoil=7,
        Npcpp=2,
        type_connection=0,
        p=3,
        Lewout=0.015,
        conductor=-1,
        init_dict=None,
        init_str=None,
    ):
        """Constructor of the class. Can be use in three ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for pyleecan type, -1 will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary with property names as keys
        - __init__ (init_str = s) s must be a string
        s is the file path to load

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for pyleecan Object"""

        if init_str is not None:  # Load from a file
            init_dict = load_init_dict(init_str)[1]
        if init_dict is not None:  # Initialisation by dict
            assert type(init_dict) is dict
            # Overwrite default value with init_dict content
            if "user_wind_mat" in list(init_dict.keys()):
                user_wind_mat = init_dict["user_wind_mat"]
            if "is_reverse_wind" in list(init_dict.keys()):
                is_reverse_wind = init_dict["is_reverse_wind"]
            if "Nslot_shift_wind" in list(init_dict.keys()):
                Nslot_shift_wind = init_dict["Nslot_shift_wind"]
            if "qs" in list(init_dict.keys()):
                qs = init_dict["qs"]
            if "Ntcoil" in list(init_dict.keys()):
                Ntcoil = init_dict["Ntcoil"]
            if "Npcpp" in list(init_dict.keys()):
                Npcpp = init_dict["Npcpp"]
            if "type_connection" in list(init_dict.keys()):
                type_connection = init_dict["type_connection"]
            if "p" in list(init_dict.keys()):
                p = init_dict["p"]
            if "Lewout" in list(init_dict.keys()):
                Lewout = init_dict["Lewout"]
            if "conductor" in list(init_dict.keys()):
                conductor = init_dict["conductor"]
        # Set the properties (value check and convertion are done in setter)
        self.user_wind_mat = user_wind_mat
        # Call Winding init
        super(WindingUD, self).__init__(
            is_reverse_wind=is_reverse_wind,
            Nslot_shift_wind=Nslot_shift_wind,
            qs=qs,
            Ntcoil=Ntcoil,
            Npcpp=Npcpp,
            type_connection=type_connection,
            p=p,
            Lewout=Lewout,
            conductor=conductor,
        )
        # The class is frozen (in Winding init), for now it's impossible to
        # add new properties

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        WindingUD_str = ""
        # Get the properties inherited from Winding
        WindingUD_str += super(WindingUD, self).__str__()
        WindingUD_str += (
            "user_wind_mat = "
            + linesep
            + str(self.user_wind_mat).replace(linesep, linesep + "\t")
            + linesep
            + linesep
        )
        return WindingUD_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False

        # Check the properties inherited from Winding
        if not super(WindingUD, self).__eq__(other):
            return False
        if not array_equal(other.user_wind_mat, self.user_wind_mat):
            return False
        return True

    def compare(self, other, name="self"):
        """Compare two objects and return list of differences"""

        if type(other) != type(self):
            return ["type(" + name + ")"]
        diff_list = list()

        # Check the properties inherited from Winding
        diff_list.extend(super(WindingUD, self).compare(other, name=name))
        if not array_equal(other.user_wind_mat, self.user_wind_mat):
            diff_list.append(name + ".user_wind_mat")
        return diff_list

    def __sizeof__(self):
        """Return the size in memory of the object (including all subobject)"""

        S = 0  # Full size of the object

        # Get size of the properties inherited from Winding
        S += super(WindingUD, self).__sizeof__()
        S += getsizeof(self.user_wind_mat)
        return S

    def as_dict(self, **kwargs):
        """
        Convert this object in a json serializable dict (can be use in __init__).
        Optional keyword input parameter is for internal use only
        and may prevent json serializability.
        """

        # Get the properties inherited from Winding
        WindingUD_dict = super(WindingUD, self).as_dict(**kwargs)
        if self.user_wind_mat is None:
            WindingUD_dict["user_wind_mat"] = None
        else:
            WindingUD_dict["user_wind_mat"] = self.user_wind_mat.tolist()
        # The class name is added to the dict for deserialisation purpose
        # Overwrite the mother class name
        WindingUD_dict["__class__"] = "WindingUD"
        return WindingUD_dict

    def _set_None(self):
        """Set all the properties to None (except pyleecan object)"""

        self.user_wind_mat = None
        # Set to None the properties inherited from Winding
        super(WindingUD, self)._set_None()

    def _get_user_wind_mat(self):
        """getter of user_wind_mat"""
        return self._user_wind_mat

    def _set_user_wind_mat(self, value):
        """setter of user_wind_mat"""
        if type(value) is int and value == -1:
            value = array([])
        elif type(value) is list:
            try:
                value = array(value)
            except:
                pass
        check_var("user_wind_mat", value, "ndarray")
        self._user_wind_mat = value

    user_wind_mat = property(
        fget=_get_user_wind_mat,
        fset=_set_user_wind_mat,
        doc=u"""user defined Winding matrix

        :Type: ndarray
        """,
    )



def get_surface_active(self, alpha=0, delta=0):
    """Return the full winding surface

    Parameters
    ----------
    self : SlotW60
        A SlotW60 object
    alpha : float
        float number for rotation (Default value = 0) [rad]
    delta : complex
        complex number for translation (Default value = 0)

    Returns
    -------
    surf_wind: Surface
        Surface corresponding to the Winding Area
    """

    raise Exception("get_surface_active not available for SlotW60")



from ....Classes.DataKeeper import DataKeeper
from ....Classes.VarLoad import VarLoad


def get_elec_datakeeper(self, symbol_list):
    """
    Generate DataKeepers to store by default results from electric module

    Parameters
    ----------
    self: VarLoadCurrent
        A VarLoadCurrent object
    symbol_list : list
        List of the existing datakeeper (to avoid duplicate)

    Returns
    -------
    dk_list: list
        list of DataKeeper
    """
    dk_list = VarLoad.get_elec_datakeeper(self, symbol_list)
    if self.type_OP_matrix == 0:  # I0 and Phi0
        if "I0" not in symbol_list:
            # Save I0
            dk_list.append(
                DataKeeper(
                    name="I0",
                    symbol="I0",
                    unit="A",
                    keeper="lambda output: np.abs(output.elec.Id_ref + 1j * output.elec.Iq_ref)",
                )
            )
        # Save Phi0
        if "Phi0" not in symbol_list:
            dk_list.append(
                DataKeeper(
                    name="Phi0",
                    symbol="Phi0",
                    unit="",
                    keeper="lambda output: np.angle(output.elec.Id_ref + 1j * output.elec.Iq_ref) % (2 * np.pi)",
                )
            )
    # Keep torque
    if self.OP_matrix.shape[1] > 3 and "Tem_av_ref" not in symbol_list:
        dk_list.append(
            DataKeeper(
                name="Reference Average Torque",
                symbol="Tem_av_ref",
                unit="N.m",
                keeper="lambda output: output.elec.Tem_av_ref",
            )
        )

    # TODO Save power
    # if self.is_power:
    #     datakeeper_list.append(
    #         DataKeeper(
    #             name="Power",
    #             symbol="Tem",
    #             unit="N.m",
    #             keeper="lambda output: output.simu.mag.power",
    #         ),
    #     )

    return dk_list



# -*- coding: utf-8 -*-


from numpy import mod, zeros, arange, sign

from ....Methods.Machine.Winding import WindingError
from ....Functions.Winding.reverse_wind_mat import reverse_wind_mat
from ....Functions.Winding.shift_wind_mat import shift_wind_mat


def comp_connection_mat(self, Zs=None):
    """Compute the Winding Matrix (for winding type 3 or 4) (Nlay_rad=1 or 2,Nlay_tan=1)
    type 3 or 4 : DISTRIBUTED SHORTED PITCH INTEGRAL WINDING

    Parameters
    ----------
    self : Winding
        A: Winding object
    Zs : int
        Number of Slot (Integer >0)

    Returns
    -------
    wind_mat: numpy.ndarray
        Winding Matrix (1 or 2, 1, Zs, qs)

    Raises
    ------
    WindingDefMsError
        Zs/2/p/qs must be an integer

    """
    if Zs is None:
        if self.parent is None:
            raise WindingError(
                "ERROR: The Winding object must be in a Lamination object."
            )

        if self.parent.slot is None:
            raise WindingError(
                "ERROR: The Winding object must be in a Lamination object with Slot."
            )

        Zs = self.parent.slot.Zs

    assert Zs > 0, "Zs must be >0"
    assert Zs % 1 == 0, "Zs must be an integer"

    coil_pitch = self.coil_pitch
    p = self.p
    nlay = self.get_dim_wind()[0]
    qs = self.qs
    ms = Zs / 2.0 / float(p) / float(qs)
    tausp = Zs / 2.0 / float(p)
    Ntcoil = self.Ntcoil  # number of turns per coils
    wind_mat = zeros((nlay, 1, Zs, qs))

    if qs == 3:
        phase_order = [1, -3, 2]
    else:
        phase_order = range(1, qs + 1)
    if ms % 1 != 0:  # if ms isn't an integer
        raise WindingDefMsError(
            "wrong winding definition, Zs/2/p/qs must " "be an integer !"
        )
    ms = int(ms)
    tausp = int(tausp)  # if ms is an integer, tausp is

    # shorted pitch Nlay-layered integral overlapping windings
    for nl in range(0, nlay):
        for i in range(0, p):
            for k in range(0, qs):
                ph = abs(phase_order[k])
                # cf Gieras p36
                z = (
                    arange(1, ms + 1)
                    + i * (Zs / p)
                    + k * ms
                    + nl * (coil_pitch - tausp)
                )

                sp = ((z - 1) % Zs).astype(int)  # positive pole
                sm = ((z + Zs / 2 / p - 1) % Zs).astype(int)  # negative pole
                for s in sp:
                    wind_mat[nl, 0, s, ph - 1] = Ntcoil * sign(
                        phase_order[k]
                    )  # Accumulation for a single slot
                for s in sm:
                    wind_mat[nl, 0, s, ph - 1] = -Ntcoil * sign(
                        phase_order[k]
                    )  # Accumulation for a single slot

    # Set default values
    if self.is_reverse_wind is None:
        self.is_reverse_wind = False
    if self.Nslot_shift_wind is None:
        self.Nslot_shift_wind = 0
    # Apply the transformations
    if self.is_reverse_wind:
        wind_mat = reverse_wind_mat(wind_mat)
    if self.Nslot_shift_wind > 0:
        wind_mat = shift_wind_mat(wind_mat, self.Nslot_shift_wind)

    return wind_mat


class WindingDefMsError(WindingError):
    """ """

    pass



# -*- coding: utf-8 -*-

from numpy import pi, sin, arcsin, abs as np_abs


def comp_surface_active(self):
    """Compute the Slot inner surface for winding (by analytical computation)

    Parameters
    ----------
    self : SlotDC
        A SlotDC object

    Returns
    -------
    Swind: float
        Slot inner surface for winding [m**2]

    """

    [
        Z1,
        Z2,
        Z3,
        Z4,
        Z5,
        Z6,
        Z7,
        Z8,
        Z9,
        Z10,
        Z11,
        Z12,
        _,
        _,
        _,
    ] = self._comp_point_coordinate()

    # Half circle R3
    S1 = self.R3 ** 2 * pi / 2
    # Tapeze H3
    S2 = self.H3 * (self.R3 * 2 + self.D2) / 2
    # Half circle D2
    S3 = (self.D2 / 2) ** 2 * pi / 2
    # Remove arc from Half Circle D2
    alpha1 = float(2 * arcsin(self.W2 / (2 * self.D2 / 2)))
    Sarc1 = ((self.D2 / 2) ** 2.0) / 2.0 * (alpha1 - sin(alpha1))
    # Rectangle Z10,Z9,Z4,Z3
    S4 = self.W2 * np_abs(Z10.real - Z9.real)
    # Circle D1
    S5 = (self.D1 / 2) ** 2 * pi
    # Remove arc from Circle D1 from rectangle Z10,Z9,Z4,Z3
    alpha2 = float(2 * arcsin(self.W2 / (2 * self.D1 / 2)))
    Sarc2 = ((self.D1 / 2) ** 2.0) / 2.0 * (alpha2 - sin(alpha2))
    # Remove arc from Circle D1 from W1 rectangle
    alpha3 = float(2 * arcsin(self.W1 / (2 * self.D1 / 2)))
    Sarc3 = ((self.D1 / 2) ** 2.0) / 2.0 * (alpha3 - sin(alpha3))

    return S1 + S2 + S3 - Sarc1 + S4 + S5 - Sarc2 - Sarc3



# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/Import/ImportGenVectLin.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/pyleecan/tree/master/pyleecan/Methods/Import/ImportGenVectLin
"""

from os import linesep
from sys import getsizeof
from logging import getLogger
from ._check import check_var, raise_
from ..Functions.get_logger import get_logger
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from .ImportMatrix import ImportMatrix

# Import all class method
# Try/catch to remove unnecessary dependencies in unused method
try:
    from ..Methods.Import.ImportGenVectLin.check import check
except ImportError as error:
    check = error

try:
    from ..Methods.Import.ImportGenVectLin.comp_step import comp_step
except ImportError as error:
    comp_step = error

try:
    from ..Methods.Import.ImportGenVectLin.get_data import get_data
except ImportError as error:
    get_data = error


from ._check import InitUnKnowClassError


class ImportGenVectLin(ImportMatrix):
    """To generate a Linspace vector"""

    VERSION = 1

    # Check ImportError to remove unnecessary dependencies in unused method
    # cf Methods.Import.ImportGenVectLin.check
    if isinstance(check, ImportError):
        check = property(
            fget=lambda x: raise_(
                ImportError("Can't use ImportGenVectLin method check: " + str(check))
            )
        )
    else:
        check = check
    # cf Methods.Import.ImportGenVectLin.comp_step
    if isinstance(comp_step, ImportError):
        comp_step = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use ImportGenVectLin method comp_step: " + str(comp_step)
                )
            )
        )
    else:
        comp_step = comp_step
    # cf Methods.Import.ImportGenVectLin.get_data
    if isinstance(get_data, ImportError):
        get_data = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use ImportGenVectLin method get_data: " + str(get_data)
                )
            )
        )
    else:
        get_data = get_data
    # save and copy methods are available in all object
    save = save
    copy = copy
    # get_logger method is available in all object
    get_logger = get_logger

    def __init__(
        self,
        start=0,
        stop=1,
        num=100,
        endpoint=True,
        is_transpose=False,
        init_dict=None,
        init_str=None,
    ):
        """Constructor of the class. Can be use in three ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for pyleecan type, -1 will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary with property names as keys
        - __init__ (init_str = s) s must be a string
        s is the file path to load

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for pyleecan Object"""

        if init_str is not None:  # Load from a file
            init_dict = load_init_dict(init_str)[1]
        if init_dict is not None:  # Initialisation by dict
            assert type(init_dict) is dict
            # Overwrite default value with init_dict content
            if "start" in list(init_dict.keys()):
                start = init_dict["start"]
            if "stop" in list(init_dict.keys()):
                stop = init_dict["stop"]
            if "num" in list(init_dict.keys()):
                num = init_dict["num"]
            if "endpoint" in list(init_dict.keys()):
                endpoint = init_dict["endpoint"]
            if "is_transpose" in list(init_dict.keys()):
                is_transpose = init_dict["is_transpose"]
        # Set the properties (value check and convertion are done in setter)
        self.start = start
        self.stop = stop
        self.num = num
        self.endpoint = endpoint
        # Call ImportMatrix init
        super(ImportGenVectLin, self).__init__(is_transpose=is_transpose)
        # The class is frozen (in ImportMatrix init), for now it's impossible to
        # add new properties

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        ImportGenVectLin_str = ""
        # Get the properties inherited from ImportMatrix
        ImportGenVectLin_str += super(ImportGenVectLin, self).__str__()
        ImportGenVectLin_str += "start = " + str(self.start) + linesep
        ImportGenVectLin_str += "stop = " + str(self.stop) + linesep
        ImportGenVectLin_str += "num = " + str(self.num) + linesep
        ImportGenVectLin_str += "endpoint = " + str(self.endpoint) + linesep
        return ImportGenVectLin_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False

        # Check the properties inherited from ImportMatrix
        if not super(ImportGenVectLin, self).__eq__(other):
            return False
        if other.start != self.start:
            return False
        if other.stop != self.stop:
            return False
        if other.num != self.num:
            return False
        if other.endpoint != self.endpoint:
            return False
        return True

    def compare(self, other, name="self"):
        """Compare two objects and return list of differences"""

        if type(other) != type(self):
            return ["type(" + name + ")"]
        diff_list = list()

        # Check the properties inherited from ImportMatrix
        diff_list.extend(super(ImportGenVectLin, self).compare(other, name=name))
        if other._start != self._start:
            diff_list.append(name + ".start")
        if other._stop != self._stop:
            diff_list.append(name + ".stop")
        if other._num != self._num:
            diff_list.append(name + ".num")
        if other._endpoint != self._endpoint:
            diff_list.append(name + ".endpoint")
        return diff_list

    def __sizeof__(self):
        """Return the size in memory of the object (including all subobject)"""

        S = 0  # Full size of the object

        # Get size of the properties inherited from ImportMatrix
        S += super(ImportGenVectLin, self).__sizeof__()
        S += getsizeof(self.start)
        S += getsizeof(self.stop)
        S += getsizeof(self.num)
        S += getsizeof(self.endpoint)
        return S

    def as_dict(self, **kwargs):
        """
        Convert this object in a json serializable dict (can be use in __init__).
        Optional keyword input parameter is for internal use only
        and may prevent json serializability.
        """

        # Get the properties inherited from ImportMatrix
        ImportGenVectLin_dict = super(ImportGenVectLin, self).as_dict(**kwargs)
        ImportGenVectLin_dict["start"] = self.start
        ImportGenVectLin_dict["stop"] = self.stop
        ImportGenVectLin_dict["num"] = self.num
        ImportGenVectLin_dict["endpoint"] = self.endpoint
        # The class name is added to the dict for deserialisation purpose
        # Overwrite the mother class name
        ImportGenVectLin_dict["__class__"] = "ImportGenVectLin"
        return ImportGenVectLin_dict

    def _set_None(self):
        """Set all the properties to None (except pyleecan object)"""

        self.start = None
        self.stop = None
        self.num = None
        self.endpoint = None
        # Set to None the properties inherited from ImportMatrix
        super(ImportGenVectLin, self)._set_None()

    def _get_start(self):
        """getter of start"""
        return self._start

    def _set_start(self, value):
        """setter of start"""
        check_var("start", value, "float")
        self._start = value

    start = property(
        fget=_get_start,
        fset=_set_start,
        doc=u"""Begin point of the linspace

        :Type: float
        """,
    )

    def _get_stop(self):
        """getter of stop"""
        return self._stop

    def _set_stop(self, value):
        """setter of stop"""
        check_var("stop", value, "float")
        self._stop = value

    stop = property(
        fget=_get_stop,
        fset=_set_stop,
        doc=u"""End point of the linspace

        :Type: float
        """,
    )

    def _get_num(self):
        """getter of num"""
        return self._num

    def _set_num(self, value):
        """setter of num"""
        check_var("num", value, "float")
        self._num = value

    num = property(
        fget=_get_num,
        fset=_set_num,
        doc=u"""Number of value in the linspace

        :Type: float
        """,
    )

    def _get_endpoint(self):
        """getter of endpoint"""
        return self._endpoint

    def _set_endpoint(self, value):
        """setter of endpoint"""
        check_var("endpoint", value, "bool")
        self._endpoint = value

    endpoint = property(
        fget=_get_endpoint,
        fset=_set_endpoint,
        doc=u"""If True, stop is the last sample. Otherwise, it is not included

        :Type: bool
        """,
    )



# -*- coding: utf-8 -*-

from numpy import exp


def rotate(self, angle):
    """Rotate the surface

    Parameters
    ----------
    self : SurfRing
        A SurfRing object

    angle : float
        the angle of rotation [rad]

    Returns
    -------
    None
    """
    # Check if the Surface is correct
    self.check()
    # rotation of every line in the Surface
    self.out_surf.rotate(angle)
    self.in_surf.rotate(angle)
    if self.point_ref is not None:
        self.point_ref = self.point_ref * exp(1j * angle)



# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/Import/ImportVectorField.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/pyleecan/tree/master/pyleecan/Methods/Import/ImportVectorField
"""

from os import linesep
from sys import getsizeof
from logging import getLogger
from ._check import check_var, raise_
from ..Functions.get_logger import get_logger
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from ._frozen import FrozenClass

# Import all class method
# Try/catch to remove unnecessary dependencies in unused method
try:
    from ..Methods.Import.ImportVectorField.get_data import get_data
except ImportError as error:
    get_data = error


from ._check import InitUnKnowClassError
from .ImportData import ImportData


class ImportVectorField(FrozenClass):
    """Abstract class for Data Import/Generation"""

    VERSION = 1

    # cf Methods.Import.ImportVectorField.get_data
    if isinstance(get_data, ImportError):
        get_data = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use ImportVectorField method get_data: " + str(get_data)
                )
            )
        )
    else:
        get_data = get_data
    # save and copy methods are available in all object
    save = save
    copy = copy
    # get_logger method is available in all object
    get_logger = get_logger

    def __init__(
        self, components=-1, name="", symbol="", init_dict=None, init_str=None
    ):
        """Constructor of the class. Can be use in three ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for pyleecan type, -1 will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary with property names as keys
        - __init__ (init_str = s) s must be a string
        s is the file path to load

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for pyleecan Object"""

        if init_str is not None:  # Load from a file
            init_dict = load_init_dict(init_str)[1]
        if init_dict is not None:  # Initialisation by dict
            assert type(init_dict) is dict
            # Overwrite default value with init_dict content
            if "components" in list(init_dict.keys()):
                components = init_dict["components"]
            if "name" in list(init_dict.keys()):
                name = init_dict["name"]
            if "symbol" in list(init_dict.keys()):
                symbol = init_dict["symbol"]
        # Set the properties (value check and convertion are done in setter)
        self.parent = None
        self.components = components
        self.name = name
        self.symbol = symbol

        # The class is frozen, for now it's impossible to add new properties
        self._freeze()

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        ImportVectorField_str = ""
        if self.parent is None:
            ImportVectorField_str += "parent = None " + linesep
        else:
            ImportVectorField_str += (
                "parent = " + str(type(self.parent)) + " object" + linesep
            )
        if len(self.components) == 0:
            ImportVectorField_str += "components = dict()" + linesep
        for key, obj in self.components.items():
            tmp = (
                self.components[key].__str__().replace(linesep, linesep + "\t")
                + linesep
            )
            ImportVectorField_str += (
                "components[" + key + "] =" + tmp + linesep + linesep
            )
        ImportVectorField_str += 'name = "' + str(self.name) + '"' + linesep
        ImportVectorField_str += 'symbol = "' + str(self.symbol) + '"' + linesep
        return ImportVectorField_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False
        if other.components != self.components:
            return False
        if other.name != self.name:
            return False
        if other.symbol != self.symbol:
            return False
        return True

    def compare(self, other, name="self"):
        """Compare two objects and return list of differences"""

        if type(other) != type(self):
            return ["type(" + name + ")"]
        diff_list = list()
        if (other.components is None and self.components is not None) or (
            other.components is not None and self.components is None
        ):
            diff_list.append(name + ".components None mismatch")
        elif self.components is None:
            pass
        elif len(other.components) != len(self.components):
            diff_list.append("len(" + name + "components)")
        else:
            for key in self.components:
                diff_list.extend(
                    self.components[key].compare(
                        other.components[key], name=name + ".components"
                    )
                )
        if other._name != self._name:
            diff_list.append(name + ".name")
        if other._symbol != self._symbol:
            diff_list.append(name + ".symbol")
        return diff_list

    def __sizeof__(self):
        """Return the size in memory of the object (including all subobject)"""

        S = 0  # Full size of the object
        if self.components is not None:
            for key, value in self.components.items():
                S += getsizeof(value) + getsizeof(key)
        S += getsizeof(self.name)
        S += getsizeof(self.symbol)
        return S

    def as_dict(self, **kwargs):
        """
        Convert this object in a json serializable dict (can be use in __init__).
        Optional keyword input parameter is for internal use only
        and may prevent json serializability.
        """

        ImportVectorField_dict = dict()
        if self.components is None:
            ImportVectorField_dict["components"] = None
        else:
            ImportVectorField_dict["components"] = dict()
            for key, obj in self.components.items():
                if obj is not None:
                    ImportVectorField_dict["components"][key] = obj.as_dict(**kwargs)
                else:
                    ImportVectorField_dict["components"][key] = None
        ImportVectorField_dict["name"] = self.name
        ImportVectorField_dict["symbol"] = self.symbol
        # The class name is added to the dict for deserialisation purpose
        ImportVectorField_dict["__class__"] = "ImportVectorField"
        return ImportVectorField_dict

    def _set_None(self):
        """Set all the properties to None (except pyleecan object)"""

        self.components = None
        self.name = None
        self.symbol = None

    def _get_components(self):
        """getter of components"""
        if self._components is not None:
            for key, obj in self._components.items():
                if obj is not None:
                    obj.parent = self
        return self._components

    def _set_components(self, value):
        """setter of components"""
        if type(value) is dict:
            for key, obj in value.items():
                if type(obj) is dict:
                    class_obj = import_class(
                        "pyleecan.Classes", obj.get("__class__"), "components"
                    )
                    value[key] = class_obj(init_dict=obj)
        if type(value) is int and value == -1:
            value = dict()
        check_var("components", value, "{ImportData}")
        self._components = value

    components = property(
        fget=_get_components,
        fset=_set_components,
        doc=u"""Dict of components (e.g. {"radial": ImportData})

        :Type: {ImportData}
        """,
    )

    def _get_name(self):
        """getter of name"""
        return self._name

    def _set_name(self, value):
        """setter of name"""
        check_var("name", value, "str")
        self._name = value

    name = property(
        fget=_get_name,
        fset=_set_name,
        doc=u"""Name of the vector field

        :Type: str
        """,
    )

    def _get_symbol(self):
        """getter of symbol"""
        return self._symbol

    def _set_symbol(self, value):
        """setter of symbol"""
        check_var("symbol", value, "str")
        self._symbol = value

    symbol = property(
        fget=_get_symbol,
        fset=_set_symbol,
        doc=u"""Symbol of the vector field

        :Type: str
        """,
    )



# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/Slot/HoleMag.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/pyleecan/tree/master/pyleecan/Methods/Slot/HoleMag
"""

from os import linesep
from sys import getsizeof
from logging import getLogger
from ._check import check_var, raise_
from ..Functions.get_logger import get_logger
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from .Hole import Hole

# Import all class method
# Try/catch to remove unnecessary dependencies in unused method
try:
    from ..Methods.Slot.HoleMag.comp_mass_magnet_id import comp_mass_magnet_id
except ImportError as error:
    comp_mass_magnet_id = error

try:
    from ..Methods.Slot.HoleMag.comp_mass_magnets import comp_mass_magnets
except ImportError as error:
    comp_mass_magnets = error

try:
    from ..Methods.Slot.HoleMag.comp_surface_magnets import comp_surface_magnets
except ImportError as error:
    comp_surface_magnets = error

try:
    from ..Methods.Slot.HoleMag.comp_volume_magnets import comp_volume_magnets
except ImportError as error:
    comp_volume_magnets = error

try:
    from ..Methods.Slot.HoleMag.get_magnet_list import get_magnet_list
except ImportError as error:
    get_magnet_list = error

try:
    from ..Methods.Slot.HoleMag.has_magnet import has_magnet
except ImportError as error:
    has_magnet = error


from ._check import InitUnKnowClassError
from .Material import Material


class HoleMag(Hole):
    """Hole with magnets for lamination (abstract)"""

    VERSION = 1

    # Check ImportError to remove unnecessary dependencies in unused method
    # cf Methods.Slot.HoleMag.comp_mass_magnet_id
    if isinstance(comp_mass_magnet_id, ImportError):
        comp_mass_magnet_id = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use HoleMag method comp_mass_magnet_id: "
                    + str(comp_mass_magnet_id)
                )
            )
        )
    else:
        comp_mass_magnet_id = comp_mass_magnet_id
    # cf Methods.Slot.HoleMag.comp_mass_magnets
    if isinstance(comp_mass_magnets, ImportError):
        comp_mass_magnets = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use HoleMag method comp_mass_magnets: "
                    + str(comp_mass_magnets)
                )
            )
        )
    else:
        comp_mass_magnets = comp_mass_magnets
    # cf Methods.Slot.HoleMag.comp_surface_magnets
    if isinstance(comp_surface_magnets, ImportError):
        comp_surface_magnets = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use HoleMag method comp_surface_magnets: "
                    + str(comp_surface_magnets)
                )
            )
        )
    else:
        comp_surface_magnets = comp_surface_magnets
    # cf Methods.Slot.HoleMag.comp_volume_magnets
    if isinstance(comp_volume_magnets, ImportError):
        comp_volume_magnets = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use HoleMag method comp_volume_magnets: "
                    + str(comp_volume_magnets)
                )
            )
        )
    else:
        comp_volume_magnets = comp_volume_magnets
    # cf Methods.Slot.HoleMag.get_magnet_list
    if isinstance(get_magnet_list, ImportError):
        get_magnet_list = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use HoleMag method get_magnet_list: " + str(get_magnet_list)
                )
            )
        )
    else:
        get_magnet_list = get_magnet_list
    # cf Methods.Slot.HoleMag.has_magnet
    if isinstance(has_magnet, ImportError):
        has_magnet = property(
            fget=lambda x: raise_(
                ImportError("Can't use HoleMag method has_magnet: " + str(has_magnet))
            )
        )
    else:
        has_magnet = has_magnet
    # save and copy methods are available in all object
    save = save
    copy = copy
    # get_logger method is available in all object
    get_logger = get_logger

    def __init__(
        self,
        Zh=36,
        mat_void=-1,
        magnetization_dict_offset=None,
        init_dict=None,
        init_str=None,
    ):
        """Constructor of the class. Can be use in three ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for pyleecan type, -1 will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary with property names as keys
        - __init__ (init_str = s) s must be a string
        s is the file path to load

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for pyleecan Object"""

        if init_str is not None:  # Load from a file
            init_dict = load_init_dict(init_str)[1]
        if init_dict is not None:  # Initialisation by dict
            assert type(init_dict) is dict
            # Overwrite default value with init_dict content
            if "Zh" in list(init_dict.keys()):
                Zh = init_dict["Zh"]
            if "mat_void" in list(init_dict.keys()):
                mat_void = init_dict["mat_void"]
            if "magnetization_dict_offset" in list(init_dict.keys()):
                magnetization_dict_offset = init_dict["magnetization_dict_offset"]
        # Set the properties (value check and convertion are done in setter)
        # Call Hole init
        super(HoleMag, self).__init__(
            Zh=Zh,
            mat_void=mat_void,
            magnetization_dict_offset=magnetization_dict_offset,
        )
        # The class is frozen (in Hole init), for now it's impossible to
        # add new properties

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        HoleMag_str = ""
        # Get the properties inherited from Hole
        HoleMag_str += super(HoleMag, self).__str__()
        return HoleMag_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False

        # Check the properties inherited from Hole
        if not super(HoleMag, self).__eq__(other):
            return False
        return True

    def compare(self, other, name="self"):
        """Compare two objects and return list of differences"""

        if type(other) != type(self):
            return ["type(" + name + ")"]
        diff_list = list()

        # Check the properties inherited from Hole
        diff_list.extend(super(HoleMag, self).compare(other, name=name))
        return diff_list

    def __sizeof__(self):
        """Return the size in memory of the object (including all subobject)"""

        S = 0  # Full size of the object

        # Get size of the properties inherited from Hole
        S += super(HoleMag, self).__sizeof__()
        return S

    def as_dict(self, **kwargs):
        """
        Convert this object in a json serializable dict (can be use in __init__).
        Optional keyword input parameter is for internal use only
        and may prevent json serializability.
        """

        # Get the properties inherited from Hole
        HoleMag_dict = super(HoleMag, self).as_dict(**kwargs)
        # The class name is added to the dict for deserialisation purpose
        # Overwrite the mother class name
        HoleMag_dict["__class__"] = "HoleMag"
        return HoleMag_dict

    def _set_None(self):
        """Set all the properties to None (except pyleecan object)"""

        # Set to None the properties inherited from Hole
        super(HoleMag, self)._set_None()



# -*- coding: utf-8 -*-

import numpy as np


def jacobian(self, point, nodes):
    """Compute jacobian, jacobian determinant and jacobian derivatives for linear triangle.

    Parameters
    ----------
    :param self : a RefElement object

    Returns
    -------

    """
    # TODO
    return None, None



# -*-- coding: utf-8 -*
def discretize(self):
    """It returns the discretize version of the Trapeze

    Parameters
    ----------
    self : Trapeze
        a Trapeze object

    Returns
    -------
    point_list : list
        list of complex coordinate of the points
    """

    # check if the Trapeze is correct
    self.check()
    # getting lines that delimit the Trapeze
    lines = self.get_lines()
    point_list = list()
    for line in lines:
        point_list.extend(line.discretize())
        point_list.pop()
    return point_list



# -*- coding: utf-8 -*-

from numpy import sin


def comp_surface(self):
    """Compute the Slot total surface (by analytical computation).
    Caution, the bottom of the Slot is an Arc

    Parameters
    ----------
    self : SlotM12
        A SlotM12 object

    Returns
    -------
    S: float
        Slot total surface [m**2]

    """

    Rbo = self.get_Rbo()

    S1 = self.H0 * self.W0

    # The bottom is an arc
    alpha = self.comp_angle_opening()
    Sarc = (Rbo ** 2.0) / 2.0 * (alpha - sin(alpha))

    # Because Slamination = S - Zs * Sslot
    if self.is_outwards():
        return S1 - Sarc
    else:
        return S1 + Sarc



from ...Classes.Material import Material


def compare_material(mat1, mat2):
    """
    compare materials except the name and the path

    Parameters
    ----------
    mat1: Material
    mat2: Material
    """

    tmp_mat1 = Material(init_dict=mat1.as_dict())
    tmp_mat2 = Material(init_dict=mat2.as_dict())
    tmp_mat1.name = ""
    tmp_mat2.name = ""
    tmp_mat1.path = ""
    tmp_mat2.path = ""

    return tmp_mat1 == tmp_mat2



# -*- coding: utf-8 -*-

import imageio
from os.path import join
from os import remove
from glob import glob
from SciDataTool import VectorField
from matplotlib.pyplot import subplots


def animate_as_gif(
    func,
    data_list,
    save_path="./",
    file_name="animated_plot.gif",
    index_var="t_index",
    index_max=50,
    index_step=1,
    component_list=None,
    **kwargs
):
    """Animate an existing plot command as a gif: animate

    Parameters
    ----------
    func : function
        plot command
    data_list : Data or [Data]
        Data object or list to plot
    save_path : str
        path where to save the gif
    file_name : str
        name of the gif file
    index_var : str
        name of the plot parameter along which to animate
    index_max : int
        maximum value of the index
    index_step : int
        step for the index (number of frames = index_max / index_step)
    component_list : list
        list of component names to plot in separate figures
    kwargs : dict
        parameters of func
    """

    with imageio.get_writer(join(save_path, file_name), mode="I") as writer:
        if isinstance(data_list, list):
            for i, data in enumerate(data_list):
                if isinstance(data, VectorField):
                    if component_list is None:  # default: extract all components
                        component_list = data.components.keys()
                    ncomp = len(component_list)
                    fig, axs = subplots(2, ncomp, tight_layout=True, figsize=(20, 10))
                    for j in range(0, index_max, index_step):
                        save_path_temp = save_path + "\\temp_" + str(j) + ".png"
                        kwargs["save_path"] = save_path_temp
                        for k, comp in enumerate(component_list):
                            func(
                                data.components[comp],
                                fig=fig,
                                subplot_index=k,
                                **kwargs
                            )
                        image = imageio.imread(save_path_temp)
                        writer.append_data(image)

                else:
                    for j in range(0, index_max, index_step):
                        save_path_temp = save_path + "\\temp_" + str(j) + ".png"
                        kwargs["save_path"] = save_path_temp
                        func(data, **kwargs)
                        image = imageio.imread(save_path_temp)
                        writer.append_data(image)

        else:
            if isinstance(data_list, VectorField):
                if component_list is None:  # default: extract all components
                    component_list = data.components.keys()
                ncomp = len(component_list)
                for j in range(0, index_max, index_step):
                    fig, axs = subplots(1, ncomp, tight_layout=True, figsize=(20, 10))
                    save_path_temp = save_path + "\\temp_" + str(j) + ".png"
                    kwargs[index_var] = j
                    kwargs["save_path"] = save_path_temp
                    for k, comp in enumerate(component_list):
                        func(
                            data_list.components[comp],
                            fig=fig,
                            subplot_index=k,
                            **kwargs
                        )
                    image = imageio.imread(save_path_temp)
                    writer.append_data(image)
            else:
                for j in range(0, index_max, index_step):
                    save_path_temp = save_path + "\\temp_" + str(j) + ".png"
                    kwargs[index_var] = j
                    kwargs["save_path"] = save_path_temp
                    func(data_list, **kwargs)
                    image = imageio.imread(save_path_temp)
                    writer.append_data(image)

    for file in glob(save_path + "\\temp_*"):
        remove(file)



# -*- coding: utf-8 -*-


def set_m2(self, value):
    """Convert the value the correct m² unit (to set in the object in m²)

    Parameters
    ----------
    self : Unit
        A Unit object
    value : float
        Value to convert

    Returns
    -------
    value : float
        value converted in [m²]
    """
    if self.unit_m2 == 1:  # Convert to mm²
        return value / 1000000
    else:
        return value



# -*- coding: utf-8 -*-

from matplotlib.patches import Patch
from matplotlib.pyplot import axis, legend

from ....Functions.init_fig import init_fig
from ....definitions import config_dict

SHAFT_COLOR = config_dict["PLOT"]["COLOR_DICT"]["SHAFT_COLOR"]


def plot(
    self,
    fig=None,
    ax=None,
    sym=1,
    alpha=0,
    delta=0,
    is_edge_only=False,
    is_show_fig=True,
):
    """Plot the Shaft in a matplotlib fig

    Parameters
    ----------
    self : Shaft
        A Shaft object
    fig : Matplotlib.figure.Figure
        existing figure to use if None create a new one
    ax : Matplotlib.axes.Axes object
        Axis on which to plot the data
    sym : int
        Symmetry factor (1= full machine, 2= half of the machine...)
    alpha : float
        Angle for rotation [rad]
    delta : complex
        Complex value for translation
    is_edge_only: bool
        To plot transparent Patches
    is_show_fig : bool
        To call show at the end of the method

    Returns
    -------
    None
    """

    (fig, axes, patch_leg, label_leg) = init_fig(fig=fig, ax=ax, shape="rectangle")
    # Get the shaft surface(s)
    surf_list = self.build_geometry(sym=sym, alpha=alpha, delta=delta)
    patches = list()
    for surf in surf_list:
        patches.extend(surf.get_patches(color=SHAFT_COLOR, is_edge_only=is_edge_only))
    axes.set_xlabel("(m)")
    axes.set_ylabel("(m)")
    axes.set_title("Shaft")
    for patch in patches:
        axes.add_patch(patch)
    axis("equal")

    # The Lamination is centered in the figure
    Lim = self.Drsh * 0.6
    axes.set_xlim(-Lim, Lim)
    axes.set_ylim(-Lim, Lim)

    # Add legend
    if not is_edge_only:
        patch_leg.append(Patch(color=SHAFT_COLOR))
        label_leg.append("Shaft")

        legend(patch_leg, label_leg)
    if is_show_fig:
        fig.show()



# -*- coding: utf-8 -*-

from numpy import arcsin, exp, pi, sqrt

from ....Classes.Arc1 import Arc1
from ....Classes.Arc3 import Arc3
from ....Classes.Segment import Segment


def build_geometry(self):
    """Compute the curve (Line) needed to plot the object.
    The ending point of a curve is the starting point of the next curve in
    the list

    Parameters
    ----------
    self : SlotW28
        A SlotW28 object

    Returns
    -------
    curve_list: list
        A list of 4 Segment and 3 Arc1

    """

    if self.is_outwards():
        rot_sign = 1  # Rotation direction for Arc1
    else:  # inward slot
        rot_sign = -1  # Rotation direction for Arc1
    point_dict = self._comp_point_coordinate()
    Z1 = point_dict["Z1"]
    Z2 = point_dict["Z2"]
    Z3 = point_dict["Z3"]
    Z4 = point_dict["Z4"]
    Z5 = point_dict["Z5"]
    Z6 = point_dict["Z6"]
    Z7 = point_dict["Z7"]
    Z8 = point_dict["Z8"]

    # Creation of curve
    curve_list = list()
    curve_list.append(Segment(Z1, Z2))
    curve_list.append(Arc1(Z2, Z3, rot_sign * self.R1, self.is_outwards()))
    curve_list.append(Segment(Z3, Z4))
    curve_list.append(Arc3(Z4, Z5, self.is_outwards()))
    curve_list.append(Segment(Z5, Z6))
    curve_list.append(Arc1(Z6, Z7, rot_sign * self.R1, self.is_outwards()))
    curve_list.append(Segment(Z7, Z8))

    return curve_list



# -*- coding: utf-8 -*-


def comp_height_active(self):
    """Compute the height of the winding area

    Parameters
    ----------
    self : SlotW24
        A SlotW24 object

    Returns
    -------
    Hwind: float
        Height of the winding area [m]

    """

    point_dict = self._comp_point_coordinate()
    Rbo = self.get_Rbo()
    return abs(Rbo - abs(point_dict["Z2"]))



# -*- coding: utf-8 -*-

from numpy import pi


def comp_surface_active(self):
    """Compute the Slot inner surface for winding (by analytical computation)

    Parameters
    ----------
    self : SlotW12
        A SlotW12 object

    Returns
    -------
    Swind: float
        Slot inner surface for winding [m**2]

    """

    S3 = 2 * self.R2 * self.H1
    S4 = pi * self.R2 ** 2 / 2.0

    return S3 + S4



# -*- coding: utf-8 -*-
from numpy import array


def comp_height_active(self, Ndisc=200):
    """Compute the height of the active area

    Parameters
    ----------
    self : Slot
        A Slot object
    Ndisc : int
        Number of point to discretize the lines

    Returns
    -------
    Hwind: float
        Height of the active area [m]

    """

    surf = self.get_surface_active()

    point_list = surf.discretize(Ndisc)
    point_list = abs(array(point_list))

    return max(point_list) - min(point_list)



# -*- coding: utf-8 -*-

pixmap_dict = {
    "End_Random": ":/images/images/MachineSetup/WindParam/End_Random.PNG",
    "End_Preform": ":/images/images/MachineSetup/WindParam/End_Preform.PNG",
    "Dim_Ext_Rotor": ":/images/images/MachineSetup/MachineDimension/Dimension_Stator_Rotor.png",
    "Dim_In_Rotor_No_Shaft": ":/images/images/MachineSetup/MachineDimension/Dimension_Rotor_Stator.png",
    "Dim_In_Rotor_Shaft": ":/images/images/MachineSetup/MachineDimension/Dimension_Shaft_Rotor_Stator.png",
    "SCIM": ":/images/images/MachineSetup/MachineType/machine_SCIM.png",
    "DFIM": ":/images/images/MachineSetup/MachineType/DFIM.png",
    "SynRM": ":/images/images/MachineSetup/MachineType/machine_SynRM.png",
    "SPMSM": ":/images/images/MachineSetup/MachineType/machine_SPMSM.png",
    "IPMSM": ":/images/images/MachineSetup/MachineType/machine_IPMSM.png",
    "WRSM": ":/images/images/MachineSetup/MachineType/WRSM.png",
    "cross": ":/images/images/icon/cross.png",
    "check": ":/images/images/icon/check.png",
    "search": ":/images/images/icon/search.png",
}



# -*- coding: utf-8 -*-

from numpy import arcsin, cos, pi, sin


def comp_surface(self):
    """Compute the Slot total surface (by analytical computation).
    Caution, the bottom of the Slot is an Arc

    Parameters
    ----------
    self : SlotW14
        A SlotW14 object

    Returns
    -------
    S: float
        Slot total surface [m**2]

    """

    Rbo = self.get_Rbo()
    point_dict = self._comp_point_coordinate()
    Z1 = point_dict["Z1"]
    Z2 = point_dict["Z2"]
    Z3 = point_dict["Z3"]
    Z4 = point_dict["Z4"]
    Z5 = point_dict["Z5"]
    Z6 = point_dict["Z6"]
    Z7 = point_dict["Z7"]
    Z8 = point_dict["Z8"]
    Z9 = point_dict["Z9"]
    W1 = abs(Z7.imag) * 2

    S1 = (W1 + self.W0) * self.H1 / 2
    S0 = self.W0 * self.H0

    # The bottom is an arc
    alpha = self.comp_angle_opening()
    Sarc = (Rbo ** 2.0) / 2.0 * (alpha - sin(alpha))

    Swind = self.comp_surface_active()

    if self.is_outwards():
        return S1 + S0 + Swind - Sarc
    else:
        return S1 + S0 + Swind + Sarc
